from base import *
from sparsematrix import *

__all__ = ['random_row_func', 'random_matrix',
           'random_matrix_fixed_column', 'eye_lil_matrix',
           ]

def random_row_func(N, p, weight=1., initseed=None):
    '''
    Returns a random connectivity ``row_func`` for use with :class:`UserComputedConnectionMatrix`
    
    Gives equivalent output to the :meth:`Connection.connect_random` method.
    
    Arguments:
    
    ``N``
        The number of target neurons.
    ``p``
        The probability of a synapse.
    ``weight``
        The connection weight (must be a single value).
    ``initseed``
        The initial seed value (for reproducible results).
    '''
    if initseed is None:
        initseed = pyrandom.randint(100000, 1000000) # replace this
    cur_row = numpy.zeros(N)
    myrange = numpy.arange(N, dtype=int)

    def row_func(i):
        pyrandom.seed(initseed + int(i))
        scirandom.seed(initseed + int(i))
        k = scirandom.binomial(N, p, 1)[0]
        cur_row[:] = 0.0
        cur_row[pyrandom.sample(myrange, k)] = weight
        return cur_row

    return row_func


# Generation of matrices
def random_matrix(n, m, p, value=1.):
    '''
    Generates a sparse random matrix with size (n,m).
    Entries are 1 (or optionnally value) with probability p.
    If value is a function, then that function is called for each
    non zero element as value() or value(i,j).
    '''
    # TODO:
    # Simplify (by using valuef)
    W = sparse.lil_matrix((n, m))
    if callable(value) and callable(p):
        if value.func_code.co_argcount == 0:
            valuef = lambda i, j:[value() for _ in j] # value function
        elif value.func_code.co_argcount == 2:
            try:
                failed = (array(value(0, arange(m))).size != m)
            except:
                failed = True
            if failed: # vector-based not possible
                log_debug('connections', 'Cannot build the connection matrix by rows')
                valuef = lambda i, j:[value(i, k) for k in j]
            else:
                valuef = value
        else:
            raise AttributeError, "Bad number of arguments in value function (should be 0 or 2)"

        if p.func_code.co_argcount == 2:
            # Check if p(i,j) is vectorisable
            try:
                failed = (array(p(0, arange(m))).size != m)
            except:
                failed = True
            if failed: # vector-based not possible
                log_debug('connections', 'Cannot build the connection matrix by rows')
                for i in xrange(n):
                    W.rows[i] = [j for j in range(m) if rand() < p(i, j)]
                    W.data[i] = list(valuef(i, array(W.rows[i])))
            else: # vector-based possible
                for i in xrange(n):
                    W.rows[i] = list((rand(m) < p(i, arange(m))).nonzero()[0])
                    W.data[i] = list(valuef(i, array(W.rows[i])))
        elif p.func_code.co_argcount == 0:
            for i in xrange(n):
                W.rows[i] = [j for j in range(m) if rand() < p()]
                W.data[i] = list(valuef(i, array(W.rows[i])))
        else:
            raise AttributeError, "Bad number of arguments in p function (should be 2)"
    elif callable(value):
        if value.func_code.co_argcount == 0: # TODO: should work with partial objects
            for i in xrange(n):
                k = random.binomial(m, p, 1)[0]
                W.rows[i] = sample(xrange(m), k)
                W.rows[i].sort()
                W.data[i] = [value() for _ in xrange(k)]
        elif value.func_code.co_argcount == 2:
            try:
                failed = (array(value(0, arange(m))).size != m)
            except:
                failed = True
            if failed: # vector-based not possible
                log_debug('connections', 'Cannot build the connection matrix by rows')
                for i in xrange(n):
                    k = random.binomial(m, p, 1)[0]
                    W.rows[i] = sample(xrange(m), k)
                    W.rows[i].sort()
                    W.data[i] = [value(i, j) for j in W.rows[i]]
            else:
                for i in xrange(n):
                    k = random.binomial(m, p, 1)[0]
                    W.rows[i] = sample(xrange(m), k)
                    W.rows[i].sort()
                    W.data[i] = list(value(i, array(W.rows[i])))
        else:
            raise AttributeError, "Bad number of arguments in value function (should be 0 or 2)"
    elif callable(p):
        if p.func_code.co_argcount == 2:
            # Check if p(i,j) is vectorisable
            try:
                failed = (array(p(0, arange(m))).size != m)
            except:
                failed = True
            if failed: # vector-based not possible
                log_debug('connections', 'Cannot build the connection matrix by rows')
                for i in xrange(n):
                    W.rows[i] = [j for j in range(m) if rand() < p(i, j)]
                    W.data[i] = [value] * len(W.rows[i])
            else: # vector-based possible
                for i in xrange(n):
                    W.rows[i] = list((rand(m) < p(i, arange(m))).nonzero()[0])
                    W.data[i] = [value] * len(W.rows[i])
        elif p.func_code.co_argcount == 0:
            for i in xrange(n):
                W.rows[i] = [j for j in range(m) if rand() < p()]
                W.data[i] = [value] * len(W.rows[i])
        else:
            raise AttributeError, "Bad number of arguments in p function (should be 2)"
    else:
        for i in xrange(n):
            k = random.binomial(m, p, 1)[0]
            # Not significantly faster to generate all random numbers in one pass
            # N.B.: the sample method is implemented in Python and it is not in Scipy
            W.rows[i] = sample(xrange(m), k)
            W.rows[i].sort()
            W.data[i] = [value] * k

    return W

def random_matrix_fixed_column(n, m, p, value=1.):
    '''
    Generates a sparse random matrix with size (n,m).
    Entries are 1 (or optionnally value) with probability p.
    The number of non-zero entries by per column is fixed: (int)(p*n)
    If value is a function, then that function is called for each
    non zero element as value() or value(i,j).
    '''
    W = sparse.lil_matrix((n, m))
    k = (int)(p * n)
    for j in xrange(m):
        # N.B.: the sample method is implemented in Python and it is not in Scipy
        for i in sample(xrange(n), k):
            W.rows[i].append(j)

    if callable(value):
        if value.func_code.co_argcount == 0:
            for i in xrange(n):
                W.data[i] = [value() for _ in xrange(len(W.rows[i]))]
        elif value.func_code.co_argcount == 2:
            for i in xrange(n):
                W.data[i] = [value(i, j) for j in W.rows[i]]
        else:
            raise AttributeError, "Bad number of arguments in value function (should be 0 or 2)"
    else:
        for i in xrange(n):
            W.data[i] = [value] * len(W.rows[i])

    return W

def eye_lil_matrix(n):
    '''
    Returns the identity matrix of size n as a lil_matrix
    (sparse matrix).
    '''
    M = sparse.lil_matrix((n, n))
    M.setdiag([1.] * n)
    return M



from brian import *

N = 100
wmax = 0.1
tau_pre = 10 * ms
tau_post = 10 * ms
delta_A_pre = 0.1 * wmax
delta_A_post = -1.2 * delta_A_pre
max_delay = 5 * ms

G = NeuronGroup(N, 'dV/dt=(1.1-V)/(10*ms):1', reset=0, threshold=1)
G.V = rand(N)

C = Connection(G, G, 'V', sparseness=0.1, weight=lambda i, j:rand()*wmax,
               delay=(0 * ms, max_delay))

### STDP
G_pre = NeuronGroup(N, 'dA_pre/dt=-A_pre/tau_pre:1')
G_post = NeuronGroup(N, 'dA_post/dt=-A_post/tau_post:1')
A_pre_recent = RecentStateMonitor(G_pre, 'A_pre', duration=max_delay)
A_post_recent = RecentStateMonitor(G_post, 'A_post', duration=max_delay)
def update_on_pre_spikes_immediate(spikes):
    if len(spikes):
        G_pre.A_pre[spikes] += delta_A_pre
def update_on_pre_spikes_delayed(spikes):
    if len(spikes):
        times_seq = C.delayvec.get_rows(spikes)
        times_seq = [max_delay - times for times in times_seq]
        A_post_delayed_seq = A_post_recent.get_past_values_sequence(times_seq)
        for i, A_post_delayed in zip(spikes, A_post_delayed_seq):
            C.W[i, :] = clip(C.W[i, :] + A_post_delayed, 0, wmax)
def update_on_post_spikes(spikes):
    if len(spikes):
        G_post.A_post[spikes] += delta_A_post
        times_seq = C.delayvec.get_cols(spikes)
        A_pre_delayed_seq = A_pre_recent.get_past_values_sequence(times_seq)
        for i, A_pre_delayed in zip(spikes, A_pre_delayed_seq):
            C.W[:, i] = clip(C.W[:, i] + A_pre_delayed, 0, wmax)
M_pre_immediate = SpikeMonitor(G, function=update_on_pre_spikes_immediate)
M_pre_delayed = SpikeMonitor(G, function=update_on_pre_spikes_delayed, delay=max_delay)
M_post = SpikeMonitor(G, function=update_on_post_spikes)


M = SpikeMonitor(G)
run(1 * second)
raster_plot(M)
show()



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

x = whitenoise(1*second)
print x.level
x.level = 10*dB
x.level += 5*dB
print x.level

x = Sound((whitenoise(1*second), whitenoise(1*second)))

print x.level

x.level = (30, 5)

x = x.atlevel((20, 18))

x = x*gain(5*dB)

print x.level

plot(x)
show()



from brian import *

duration = 1 * second
defaultclock.dt = 0.1 * ms

max_groups = 5
max_connections = 5
N_linear_network = [100, 200, 300, 1000, 2000, 4000]
cachemiss_array_size = 300000 # should take up 2MB
N_multivar_linear_network = 1000
vars_linear_network = range(1, 6)
N_spiking_network = 1000
rates_spiking_network = [x * Hz for x in randint(20, 300, size=10)]
fixed_syn_spiking_network = 500
syn_spiking_network = list(randint(100, 1000, size=10))#+list(randint(1000,5000,size=10))
rate_syn = 50 * Hz
N_threshold = randint(100, 1000, size=20)



'''
Implementation example of the dual resonance nonlinear (DRNL) filter with
parameters fitted for human as described in Lopez-Paveda, E. and Meddis, R., A
human nonlinear cochlear filterbank, JASA 2001.

A class called :class:`~brian.hears.DRNL` implementing this model is available
in the library.

The entire pathway consists of the sum of a linear and a nonlinear pathway.

The linear path consists of a bank of bandpass filters (second order gammatone),
a low pass function, and a gain/attenuation factor, g, in a cascade.

The nonlinear path is  a cascade consisting of a bank of gammatone filters, a
compression function, a second bank of gammatone filters, and a low
pass function, in that order.

The parameters are given in the form ``10**(p0+mlog10(cf))``.
'''
from brian import *
from brian.hears import *

simulation_duration = 50*ms
samplerate = 50*kHz
level = 50*dB  # level of the input sound in rms dB SPL
sound = whitenoise(simulation_duration, samplerate).ramp()
sound.level = level
 
nbr_cf = 50  #number of centre frequencies
#center frequencies with a spacing following an ERB scale
center_frequencies = erbspace(100*Hz,1000*Hz, nbr_cf)

#conversion to stape velocity (which are the units needed by the following centres)
sound = sound*0.00014

#### Linear Pathway ####

#bandpass filter (second order gammatone filter)
center_frequencies_linear = 10**(-0.067+1.016*log10(center_frequencies))
bandwidth_linear = 10**(0.037+0.785*log10(center_frequencies))
order_linear = 3
gammatone = ApproximateGammatone(sound, center_frequencies_linear,
                                 bandwidth_linear, order=order_linear)

#linear gain
g = 10**(4.2-0.48*log10(center_frequencies))
func_gain = lambda x:g*x
gain = FunctionFilterbank(gammatone, func_gain)

#low pass filter(cascade of 4 second order lowpass butterworth filters)
cutoff_frequencies_linear = center_frequencies_linear
order_lowpass_linear = 2
lp_l = LowPass(gain, cutoff_frequencies_linear)
lowpass_linear = Cascade(gain, lp_l, 4)

#### Nonlinear Pathway ####

#bandpass filter (third order gammatone filters)
center_frequencies_nonlinear = center_frequencies
bandwidth_nonlinear = 10**(-0.031+0.774*log10(center_frequencies))
order_nonlinear = 3
bandpass_nonlinear1 = ApproximateGammatone(sound, center_frequencies_nonlinear,
                                           bandwidth_nonlinear,
                                           order=order_nonlinear)

#compression (linear at low level, compress at high level)
a = 10**(1.402+0.819*log10(center_frequencies))  #linear gain
b = 10**(1.619-0.818*log10(center_frequencies))  
v = .2 #compression exponent
func_compression = lambda x: sign(x)*minimum(a*abs(x), b*abs(x)**v)
compression = FunctionFilterbank(bandpass_nonlinear1, func_compression)

#bandpass filter (third order gammatone filters)
bandpass_nonlinear2 = ApproximateGammatone(compression,
                                           center_frequencies_nonlinear,
                                           bandwidth_nonlinear,
                                           order=order_nonlinear)

#low pass filter
cutoff_frequencies_nonlinear = center_frequencies_nonlinear
order_lowpass_nonlinear = 2
lp_nl = LowPass(bandpass_nonlinear2, cutoff_frequencies_nonlinear)
lowpass_nonlinear = Cascade(bandpass_nonlinear2, lp_nl, 3)

#adding the two pathways
dnrl_filter = lowpass_linear+lowpass_nonlinear

dnrl = dnrl_filter.process()

figure()
imshow(flipud(dnrl.T), aspect='auto')    
show()



'''
Input-Frequency curve of a neuron (cortical RS type)
Network: 1000 unconnected integrate-and-fire neurons (Brette-Gerstner)
with an input parameter I.
The input is set differently for each neuron.
Spikes are sent to a 'neuron' group with the same size and variable n,
which has the role of a spike counter.
'''
from brian import *
from brian.library.IF import *

N = 1000
eqs = Brette_Gerstner() + Current('I:amp')
print eqs
group = NeuronGroup(N, model=eqs, threshold= -20 * mV, reset=AdaptiveReset())
group.vm = -70 * mV
group.I = linspace(0 * nA, 1 * nA, N)

counter = NeuronGroup(N, model='n:1')
C = IdentityConnection(group, counter, 'n')

i = N * 8 / 10
trace = StateMonitor(group, 'vm', record=i)

duration = 5 * second
run(duration)
subplot(211)
plot(group.I / nA, counter.n / duration)
xlabel('I (nA)')
ylabel('Firing rate (Hz)')
subplot(212)
plot(trace.times / ms, trace[i] / mV)
xlabel('Time (ms)')
ylabel('Vm (mV)')
show()



from brian import *
from brian.library.IF import *

inputGroup = PoissonGroup(500, 54)
Mall = SpikeMonitor(inputGroup)
run(5000 * ms)

# XXX

eqs = Izhikevich(a=0.02 / ms, b=0.2 / ms)
Ginput = SpikeGeneratorGroup(500, Mall.spikes)
Goutput = NeuronGroup(1, model=eqs, threshold=30 * mV,
            reset=AdaptiveReset(Vr= -65 * mV, b=0.0805 * nA))

Mvolt = StateMonitor(Goutput, 'vm', record=0)

C = Connection(Ginput, Goutput, state='vm')
C.connect_full(weight=0.475 * mV)
Ginput.v = -65 * mV
Goutput.v = -65 * mV

stdp = ExponentialSTDP(C, 33 * ms, 16.8 * ms, 0.01, -0.01, wmax=0.01)

reinit_default_clock()
run(5000 * ms)

plot(Mvolt.times / (1 * msecond), Mvolt[0] / (1 * mvolt))
show()



'''
This example implements a band pass filter whose center frequency is modulated
by an Ornstein-Uhlenbeck. The white noise term used for this process is output
by a FunctionFilterbank. The bandpass filter coefficients update is an example
of how to use a :class:`~brian.hears.ControlFilterbank`. The bandpass filter is
a basic biquadratic filter for which the Q factor and the center frequency must
be given. The input is a white noise.
'''

from brian import *
from brian.hears import *

samplerate = 20*kHz
SoundDuration = 300*ms
sound = whitenoise(SoundDuration, samplerate).ramp() 

#number of frequency channel (here it must be one as a spectrogram of the
#output is plotted)
nchannels = 1   

fc_init = 5000*Hz   #initial center frequency of the band pass filter
Q = 5               #quality factor of the band pass filter
update_interval = 4 # the filter coefficients are updated every 4 samples

#parameters of the Ornstein-Uhlenbeck process
s_i = 1200*Hz
tau_i = 100*ms      
mu_i = fc_init/tau_i
sigma_i = sqrt(2)*s_i/sqrt(tau_i)
deltaT = defaultclock.dt

#this function  is used in a FunctionFilterbank. It outputs a noise term that
#will be later used by the controler to update the center frequency
noise = lambda x: mu_i*deltaT+sigma_i*randn(1)*sqrt(deltaT)
noise_generator = FunctionFilterbank(sound, noise)

#this class will take as input the output of the noise generator and as target
#the bandpass filter center frequency
class CoeffController(object):
    def __init__(self, target):
        self.target = target
        self.deltaT = 1./samplerate
        self.BW = 2*arcsinh(1./2/Q)*1.44269
        self.fc = fc_init
        
    def __call__(self, input):
        #the control variables are taken as the last of the buffer
        noise_term = input[-1,:]
        #update the center frequency by updateing the OU process
        self.fc = self.fc-self.fc/tau_i*self.deltaT+noise_term

        w0 = 2*pi*self.fc/samplerate
        #update the coefficient of the biquadratic filterbank
        alpha = sin(w0)*sinh(log(2)/2*self.BW*w0/sin(w0))
        self.target.filt_b[:, 0, 0] = sin(w0)/2
        self.target.filt_b[:, 1, 0] = 0
        self.target.filt_b[:, 2, 0] = -sin(w0)/2
    
        self.target.filt_a[:, 0, 0] = 1+alpha
        self.target.filt_a[:, 1, 0] = -2*cos(w0)
        self.target.filt_a[:, 2, 0] = 1-alpha
        
# In the present example the time varying filter is a LinearFilterbank therefore
#we must initialise the filter coefficients; the one used for the first buffer computation
w0 = 2*pi*fc_init/samplerate
BW = 2*arcsinh(1./2/Q)*1.44269
alpha = sin(w0)*sinh(log(2)/2*BW*w0/sin(w0))

filt_b = zeros((nchannels, 3, 1))
filt_a = zeros((nchannels, 3, 1))
filt_b[:, 0, 0] = sin(w0)/2
filt_b[:, 1, 0] = 0
filt_b[:, 2, 0] = -sin(w0)/2
filt_a[:, 0, 0] = 1+alpha
filt_a[:, 1, 0] = -2*cos(w0)
filt_a[:, 2, 0] = 1-alpha

#the filter which will have time varying coefficients
bandpass_filter = LinearFilterbank(sound, filt_b, filt_a)
#the updater
updater = CoeffController(bandpass_filter)

#the controller. Remember it must be the last of the chain
control = ControlFilterbank(bandpass_filter, noise_generator, bandpass_filter,
                            updater, update_interval)          

time_varying_filter_mon = control.process()

figure(1)
pxx, freqs, bins, im = specgram(squeeze(time_varying_filter_mon),
                                NFFT=256, Fs=samplerate, noverlap=240) 
imshow(flipud(pxx), aspect='auto')

show()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
"""
Functions which check the dimensions of their arguments, etc.

Functions updated to provide Quantity functionality
---------------------------------------------------

With any dimensions:

* sqrt

Dimensionless:

* log, exp
* sin, cos, tan
* arcsin, arccos, arctan
* sinh, cosh, tanh
* arcsinh, arccosh, arctanh

With homogeneous dimensions:

* dot
"""

from brian_unit_prefs import bup
from units import *
import numpy, math, scipy
from numpy import *
from numpy.random import *
from scipy.integrate import *
import inspect

__all__ = []

# these functions are the ones that will work with the template immediately below, and
# extend the numpy functions to know about Quantity objects 
quantity_versions = [
         'sqrt',
         'log', 'exp',
         'sin', 'cos', 'tan',
         'arcsin', 'arccos', 'arctan',
         'sinh', 'cosh', 'tanh',
         'arcsinh', 'arccosh', 'arctanh'
         ]

def make_quantity_version(func):
    funcname = func.__name__
    def f(x):
        if isinstance(x, Quantity):
            return getattr(x, funcname)()
        return func(x)
    f.__name__ = func.__name__
    f.__doc__ = func.__doc__
    if hasattr(func, '__dict__'):
        f.__dict__.update(func.__dict__)
    return f

for name in quantity_versions:
    if bup.use_units:
        exec name + '=make_quantity_version(' + name + ')'
    __all__.append(name)



'''
A client script to be run for cluster simulations.
'''
from brian import *
import pypar

__all__ = ['run_client']


class ClientNetwork(Network):
    '''
    Network class for running a simulation over a cluster, client side.
    '''
    pass

def run_client():
    '''
    Runs
    '''
    # Identification
    myid = pypar.rank() # id of this process
    nproc = pypar.size() # number of processors

    print "I am client", myid
    pypar.finalize()

if __name__ == '__main__':
    run_client()



#!/usr/bin/env python
#import brian_no_units
from brian import *
from brian.library.IF import *
from brian.experimental.general_stdp import *
from brian.experimental.heterog_delay import *
from brian.experimental.stdp_sparse import *
from brian.connection import DenseConnectionMatrix
import numpy
import time, os
import pickle

set_global_preferences(useweave=True)

defaultclock.dt = 0.5 * ms


class VariableIzhikevichReset(object):
    def __call__(self, P):
        spikes = P.LS.lastspikes()
        P.vm_[spikes] = Vr
        P.w_[spikes] += P.d_[spikes]

T = 100 * ms

#Network parameters
N = 1000 # total number of neurons
Ne = 800 # number of excitatory neurons
Ni = 200 # number of inhibitory neurons
Nc = 100 # number of synapses per neuron

vm0 = -65 * mV # the initial membrane current
w0 = 0.2 * vm0 / ms # the initial recovery varible
Vr = -65 * mV
Vt = 30 * mV # "firing" threshold
min_delay = 1 * ms
max_delay = 20 * ms # maximum axonal delay
i_delay = 1 * ms # the inhibitory delay
wE0 = 6 * mV #initial positive synaptic weight
wI0 = -5 * mV #initial negative synaptic weight
# I think these are wrong, the units should be mV
#wE0= 6*nA #initial positive synaptic weight
#wI0= -5*nA #initial negative synaptic weight

#STDP
gmax = 10 * mV#0.010 # maximum synaptic weight
tau_pre = 20 * ms
tau_post = 20 * ms
dA_pre = gmax * .005
dA_post = -dA_pre * 1.05

if os.path.exists('izhikevich.pkl'):
    print "Loading saved network from file..."
    inputfile = open('izhikevich.pkl', 'rb')
    objs = pickle.load(inputfile)
    inputfile.close()
    clk, model, G, Ce, Cie, S, net = objs

else:
    eqs = Izhikevich(a='a', b=0.2 / ms) + '''
            a : Hz
            d : volt/second
            '''
    model = Model(model=eqs, threshold=Vt, reset=VariableIzhikevichReset())

    #dt bit needed for versions of Brian which aren't recent
    G = NeuronGroup(N, model, max_delay=i_delay + defaultclock.dt, compile=True, freeze=True)

    Ge = G.subgroup(Ne)
    Gi = G.subgroup(Ni)

    Ge.vm = vm0
    Ge.w = w0
    Ge.a = 0.02 / ms
    Ge.d = 8.0 * mV / ms

    Gi.vm = vm0
    Gi.w = w0
    Gi.a = 0.1 / ms
    Gi.d = 2.0 * mV / ms

    start_time = time.time()
    Ce = HeterogeneousDelayConnection(Ge, G, max_delay=max_delay, delays_per_synapse=True)
    #Cei = HeterogeneousDelayConnection(Ge, Gi, max_delay=max_delay, delays_per_synapse=True)
    Cie = Connection(Gi, Ge, delay=1 * ms)

    # Add Nc connections to each neuron in the network
    #Ce.connect_random(Ge, Ge, p=float(Nc)/N, weight=wE0)
    #Ce.connect_random(Ge, Gi, p=float(Nc)/N, weight=wI0)#really want this to be wI0?
    Ce.connect_random(Ge, G, p=float(Nc) / N, weight=wE0)
    for i in range(len(Ge)):
        Ce[i, i] = 0
    Ce.delayvec = DenseConnectionMatrix((len(Ge), len(G))) # faster in the current implementaton I think

    # This way is slow, but if we're making delayvec a sparse matrix we have to do it like this
    #for i in xrange(len(Ge)):
    #    for j in xrange(len(G)):
    #        if abs(Ce[i,j])>1e-30:
    #            Ce.delayvec[i,j]=rand()*(max_delay-min_delay)+min_delay

    Ce.delayvec[:] = random((len(Ge), len(G))) * (max_delay - min_delay) + min_delay
    # Use this if you need to be sure they are integer multiples of 1ms, but not necessary I think
    #Ce.delayvec[:] = ((random((len(Ge), len(G)))*max_delay/ms).astype(int)).astype(float)*ms

    # An alternative idea: SparseSTDPConnectionMatrix faster for delayvec too? Answer: not noticeably
    #Ce.delayvec = SparseSTDPConnectionMatrix(Ce.W)
    #Ce.delayvec[:] = rand(len(Ce.delayvec[:]))*(max_delay-min_delay)+min_delay
    #Ce.delayvec.ndim = 2

    Cie.connect_random(Gi, Ge, p=float(Nc) / N, weight=wI0)

    # if you turn STDP off it takes 20 times longer! The SparseSTDPConnectionMatrix is MORE efficient than
    # what was there before!!
    STDPe = SongAbbottSTDP(Ce, gmax=gmax, tau_pre=tau_pre, tau_post=tau_post,
                          dA_pre=dA_pre, dA_post=dA_post)
    Ce.W = SparseSTDPConnectionMatrix(Ce.W)

    print "Network build time:", time.time() - start_time, "seconds"

    thalamic_alternate = True

    Gvm_ = G.vm_
    thaldelta = float(20 * mV)

    S = SpikeMonitor(G)


@network_operation(when='start')
def thalamic_input():
    global thalamic_alternate
    # at each cycle, some random neuron gets a 20*mV boost
    if thalamic_alternate:
        n = int(N * random())
        Gvm_[n] += thaldelta#float(20*mV)
        thalamic_alternate = False
    else:
        thalamic_alternate = True

net = MagicNetwork()
#net.update_schedule_groups_resets_connections()

def do_run(T=T, doplot=True):
    start_time = time.time()
    print "Running the Izhikevich spnet simulation for %s:" % display_in_unit(T, second)
    net.run(T)
    print "Excecution time:", time.time() - start_time, "seconds"
    if doplot:
        raster_plot(S)
        show()

do_run()

# save the network to a file
clk = defaultclock
obj = (clk, model, G, Ce, Cie, S, net)
output = open('izhikevich.pkl', 'wb')
pickle.dump(obj, output, -1)
output.close()

#import cProfile
#cProfile.run('do_run(1000*ms,False)')

#import pycallgraph
#cg_func = 'Network.update'
#def ff(pat):
#    def f(call_stack, module_name, class_name, func_name, full_name):
#        if not 'brian' in module_name: return False
#        for n in call_stack+[full_name]:
#            if pat in n:
#                return True
#        return False
#    return f
#pycallgraph.start_trace(filter_func=ff(cg_func))
#do_run(1*second,False)
#pycallgraph.stop_trace()
#pycallgraph.make_dot_graph('callgraph-'+cg_func+'.png')



'''
Example of basic use and manipulation of sounds with Brian hears.
'''
from brian import *
from brian.hears import *

sound1 = tone(1*kHz, 1*second)
sound2 = whitenoise(1*second)

sound = sound1+sound2
sound = sound.ramp()

# Comment this line out if you don't have pygame installed
sound.play()

# The first 20ms of the sound
startsound = sound[:20*ms]

subplot(121)
plot(startsound.times, startsound)
subplot(122)
sound.spectrogram()
show()



from brian import *
from brian.hears import *

x = whitenoise(100*ms)
fb = Gammatone(x, [1*kHz, 1.01*kHz])

#fb.duration = 2*second

print fb.duration

nsamples = x.nsamples

def sum_of_squares(input, running_sum_of_squares):
    return running_sum_of_squares+sum(input**2, axis=0)
rms = sqrt(fb.process(sum_of_squares)/nsamples)

print rms

y = fb.process()
y = fb.process()

print sqrt(sum(y**2, axis=0)/nsamples)

plot(y)
show()



"""

This appears to be an accurate model of the time spent in the threshold
calling mechanism, and so can be used as a basis for optimising.

With netsize = 4000 and repeats = 400000 as in the cuba-profile.txt example,
using original method gives 40s in nonzero method of numpy.ndarray, which
accords with what happens in the CUBA profile, and 11s in
FakeThreshold.__call__ which also accords.

"""

from numpy import *
import c_profile
import time

## parameters for comparison with cuba-profile.txt
#netsize = 4000
#repeats = 400000
# parameters for quicker runs to try out optimisations
# original method: 4.768s in nonzero, 1.366 in __call__
netsize = 4000
repeats = 50000


class FakeNeuronGroup(object):
    pass


class FakeThreshold(object):
    # this is the original method
    def __call__(self, P):
        return ((P.S[0] > self.Vt).nonzero())[0]

V = random.uniform(size=netsize)
Vt = 1 - 0.0005

P = FakeNeuronGroup()
for i in range(40):
    exec 'P.a' + str(i) + '=1'
P.S = zeros((3, netsize))
P.S[0] = V

T = FakeThreshold()
#T.Vt = Vt
for i in range(40):
    exec 'T.a' + str(i) + '=1'

def main():
    for i in xrange(repeats):
        spikes = T(P)
c_profile.run('main()')

#def timeforfiringfraction(Vt):
#    global T
#    T.Vt = Vt
#    start = time.time()
#    for i in xrange(repeats):
#        spikes = T(P)
#    return time.time()-start
#
#def main():
#    for rate in [1,5,10,50,100,400,4000]:
#        Vt = 1 - float(rate)/4000.
#        print rate, ",", timeforfiringfraction(Vt)
#        
#c_profile.run('main()')



'''
Model fitting example.
Fit an integrate-and-fire model to an in-vitro electrophysiological 
recording during one second.
'''
from brian import loadtxt, ms, Equations
from brian.library.modelfitting import *

if __name__ == '__main__':
    nvar = 100
    equations = '''
        dV/dt=(R*I-V)/tau : 1
        I : 1
        R : 1
        tau : second
    '''
    equations += '\n'.join(['var'+str(i)+':0' for i in range(nvar)])
    equations = Equations(equations)
    input = loadtxt('current.txt')
    spikes = loadtxt('spikes.txt')
    results = modelfitting( model = equations,
                            reset = 0,
                            threshold = 1,
                            data = spikes,
                            input = input,
                            dt = .1*ms,
                            popsize = 1000,
                            maxiter = 3,
                            delta = 4*ms,
                            gpu = 1,
                            R = [1.0e9, 9.0e9],
                            tau = [10*ms, 40*ms],
                            refractory = [0*ms, 10*ms])
    print_table(results)



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
'''
Differential equations for Brian models.
'''
#from scipy.weave import blitz
from operator import isSequenceType
import types
from units import *
from stdunits import *
from inspection import *
from scipy import exp
from scipy import weave
from globalprefs import *
import re
import inspect
import optimiser
import warnings
import uuid
import numpy
from numpy import zeros, ones
import numpy
from log import *
from optimiser import *
from scipy import optimize
import unitsafefunctions
import copy
try:
    import sympy
    use_sympy = True
except:
    warnings.warn('sympy not installed')
    use_sympy = False

__all__ = ['Equations', 'unique_id']

# TODO: write interface for equations.py

def unique_id():
    """
    Returns a unique name (e.g. for internal hidden variables).
    """
    return '_' + str(uuid.uuid1().int)


class Equations(object):
    """Container that stores equations from which models can be created
    
    Initialised as::
    
        Equations(expr[,level=0[,keywords...]])
    
    with arguments:
    
    ``expr``
        An expression, which can each be a string representing equations,
        an :class:`Equations` objects, or a list of strings and :class:`Equations` objects.
        See below for details of the string format.
    ``level``
        Indicates how many levels back in the stack the namespace for string
        equations is found, so that e.g. ``level=0`` looks in the
        namespace of the function where the :class:`Equations` object was created,
        ``level=1`` would look in the namespace of the function that called the
        function where the :class:`Equations` object was created, etc.
        Normally you can just leave this out.
    ``keywords``
        Any sequence of keyword pairs ``key=value`` where the string ``key``
        in the string equations will be replaced with ``value`` which can
        be either a string, value or ``None``, in the latter case a unique
        name will be generated automatically (but it won't be pretty).
    
    Systems of equations can be defined by passing lists of :class:`Equations` to a
    new :class:`Equations` object, or by adding :class:`Equations` objects together (the usage
    is similar to that of a Python ``list``).
    
    **String equations**
    
    String equations can be of any of the following forms:
    
    (1) ``dx/dt = f : unit`` (differential equation)
    (2) ``x = f : unit`` (equation)
    (3) ``x = y`` (alias)
    (4) ``x : unit`` (parameter)
    
    Here each of ``x`` and ``y`` can be any valid Python variable name,
    ``f`` can be any valid Python expression, and ``unit`` should be the
    unit of the corresponding ``x``. You can also include multi-line
    expressions by appending a ``\`` character at the end of each line
    which is continued on the next line (following the Python standard),
    or comments by including a ``#`` symbol.
    
    These forms mean:
    
    *Differential equation*
        A differential equation with variable ``x`` which has physical
        units ``unit``. The variable ``x`` will become one of the state
        variables of the model.
    *Equation*
        An equation defining the meaning of ``x`` can be used for building
        systems of complicated differential equations.
    *Alias*
        The variable ``x`` becomes equivalent to the variable ``y``, useful
        for connecting two separate systems of equations together.
    *Parameter*
        The variable ``x`` will have physical units ``unit`` and will be
        one of the state variables of the model (but will not evolve
        dynamically, instead it should be set by the user).
        
    .. index::
        single: xi
        pair: xi; noise
        single: white noise
        single: gaussian noise
        single: noise
        single: noise; gaussian
        single: noise; white
    
    **Noise**
        
    String equations can also use the reserved term ``xi`` for a
    Gaussian white noise with mean 0 and variance 1.
    
    **Example usage** ::

        eqs=Equations('''
        dv/dt=(u-v)/tau : volt
        u=3*v : volt
        w=v
        ''')
    
    **Details**
    
    For more details, see :ref:`moreonequations` in the user manual.
    """
    def __init__(self, expr='', level=0, **kwds):
        # Empty object
        self._Vm = None # name of variable with membrane potential
        self._eq_names = [] # equations names
        self._diffeq_names = [] # differential equations names
        self._diffeq_names_nonzero = [] # differential equations names
        self._function = {} # dictionary of functions
        self._string = {} # dictionary of strings (defining the functions)
        self._namespace = {} # dictionary of namespaces for the strings (globals,locals)
        self._alias = {} # aliases (mapping name1 -> name2)
        self._units = {'t':second} # dictionary of units
        self._dependencies = {} # dictionary of dependencies (on static equations)
        self._useweave = get_global_preference('useweave')
        self._cpp_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._cpp_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']
        self._frozen = False # True if all units and parameters are gone
        self._prepared = False

        if not isinstance(expr, str): # assume it is a sequence of Equations objects
            for eqs in expr:
                if not isinstance(eqs, Equations):
                    eqs = Equations(eqs, level=level + 1)
                self += eqs
        elif expr != '':
            # Check keyword arguments
            param_dict = {}
            for name, value in kwds.iteritems():
                if value is None: # name is not important: choose unique name
                    value = unique_id()
                if isinstance(value, str): # variable name substitution
                    expr = re.sub('\\b' + name + '\\b', value, expr)
                    expr = re.sub('\\bd' + name + '\\b', 'd' + value, expr) # derivative
                else:
                    param_dict[name] = value

            if kwds == {}: # weird: changed from param_dict on 18/06/08
                self.parse_string_equations(expr, level=level + 1)
            else:
                self.parse_string_equations(expr, namespace=param_dict, level=level + 1)

    """
    -----------------------------------------------------------------------
    PARSING AND BUILDING NAMESPACES
    -----------------------------------------------------------------------
    """

    def parse_string_equations(self, eqns, level=1, namespace=None):
        """
        Parses a string defining equations and builds an Equations object.
        Uses the namespace in the given level of the stack.
        """
        diffeq_pattern = re.compile('\s*d(\w+)\s*/\s*dt\s*=\s*(.+?)\s*:\s*(.*)')
        eq_pattern = re.compile('\s*(\w+)\s*=\s*(.+?)\s*:\s*(.*)')
        alias_pattern = re.compile('\s*(\w+)\s*=\s*(\w+)\s*$')
        param_pattern = re.compile('\s*(\w+)\s*:\s*(.*)')
        empty_pattern = re.compile('\s*$')
        patterns = [diffeq_pattern, eq_pattern, alias_pattern, param_pattern, empty_pattern]
        # Merge multi-line statements
        eqns = re.sub('\\\s*?\n', ' ', eqns)

        # Namespace of the functions
        ns_global, ns_local = namespace, namespace
        if namespace is None:
            frame = inspect.stack()[level + 1][0]
            ns_global, ns_local = frame.f_globals, frame.f_locals
            #print frame.f_code.co_filename #useful for debugging which file the namespace came from

        for line in eqns.splitlines():
            line = re.sub('#.*', '', line) # remove comments
            result = None
            for pattern in patterns:
                result = pattern.match(line)
                if result:
                    break
            if result == None:
                raise TypeError, "Invalid equation string: " + line
            if pattern == eq_pattern:
                name, eq, unit = result.groups()
                self.add_eq(name, eq, unit, ns_global, ns_local)
            elif pattern == diffeq_pattern:
                name, eq, unit = result.groups()
                self.add_diffeq(name, eq, unit, ns_global, ns_local)
            elif pattern == alias_pattern:
                name1, name2 = result.groups()
                self.add_alias(name1, name2)
            elif pattern == param_pattern:
                name, unit = result.groups()
                self.add_param(name, unit, ns_global, ns_local)

    def add_eq(self, name, eq, unit, global_namespace={}, local_namespace={}):
        """
        Inserts an equation.
        name = variable name
        eq = string definition
        unit = unit of the variable (possibly a string)
        *_namespace = namespaces associated to the string
        """
        # Find external objects
        vars = list(get_identifiers(eq))
        if type(unit) == types.StringType:
            vars.extend(list(get_identifiers(unit)))
        self._namespace[name] = {}
        for var in vars:
            if var in local_namespace: #local
                self._namespace[name][var] = local_namespace[var]
            elif var in global_namespace: #global
                self._namespace[name][var] = global_namespace[var]
            elif var in globals(): # typically units
                self._namespace[name][var] = globals()[var]

        self._eq_names.append(name)
        if type(unit) == types.StringType:
            self._units[name] = eval(unit, self._namespace[name].copy())
        else:
            self._units[name] = unit
        self._string[name] = eq

    def add_diffeq(self, name, eq, unit, global_namespace={}, local_namespace={}, nonzero=True):
        """
        Inserts a differential equation.
        name = variable name
        eq = string definition
        unit = unit of the variable (possibly a string)
        *_namespace = namespaces associated to the string
        nonzero = False if dx/dt=0 (parameter)
        """
        # Find external objects
        vars = list(get_identifiers(eq))
        if type(unit) == types.StringType:
            vars.extend(list(get_identifiers(unit)))
        self._namespace[name] = {}
        for var in vars:
            if var in local_namespace: #local
                self._namespace[name][var] = local_namespace[var]
            elif var in global_namespace: #global
                self._namespace[name][var] = global_namespace[var]
            elif var in globals(): # typically units
                self._namespace[name][var] = globals()[var]

        self._diffeq_names.append(name)
        if type(unit) == types.StringType:
            self._units[name] = eval(unit, self._namespace[name].copy())
        else:
            self._units[name] = unit
        self._string[name] = eq
        if nonzero:
            self._diffeq_names_nonzero.append(name)

    def add_alias(self, name1, name2):
        """
        Inserts an alias.
        name1 = new name
        name2 = old name
        """
        self._alias[name1] = name2
        # TODO: what if name2 is not defined yet?
        self.add_eq(name1, name2, self._units[name2])

    def add_param(self, name, unit, global_namespace={}, local_namespace={}):
        """
        Inserts a parameter.
        name = variable name
        eq = string definition
        unit = unit of the variable (possibly a string)
        *_namespace = namespaces associated to the string
        """
        if isinstance(unit, Quantity):
            unit = scalar_representation(unit)
        self.add_diffeq(name, '0*' + unit + '/second', unit, global_namespace, local_namespace, nonzero=False)

    """
    -----------------------------------------------------------------------
    FINALISATION
    -----------------------------------------------------------------------
    """

    def prepare(self, check_units=True):
        '''
        Do a number of checks (units) and preparation of the object.
        '''
        if self._prepared:
            return
        # Let Vm be the first differential equation
        vm_name = self.get_Vm()
        if vm_name:
            # TODO: INFO logging
            i = self._diffeq_names.index(vm_name)
            self._diffeq_names[0], self._diffeq_names[i] = self._diffeq_names[i], self._diffeq_names[0]
        else:
            pass
            # TODO: WARNING log that a potential problem has occurred here?

        # Clean namespace (avoids conflicts between variables and external variables)
        self.clean_namespace()
        # Compile strings to functions
        self.compile_functions()
        # Check units
        if check_units: self.check_units()
        # Set the update order of (static) variables
        self.set_eq_order()
        # Replace static variables by their value in differential equations
        self.substitute_eq()
        self.compile_functions()

        # Check free variables
        free_vars = self.free_variables()
        if free_vars != []:
            log_info('brian.equations', 'Free variables: ' + str(free_vars))

        self._prepared = True

    def get_Vm(self):
        '''
        Finds the variable that is most likely to be the
        membrane potential.
        '''
        if self._Vm:
            return self._Vm
        vm_names = ['v', 'V', 'vm', 'Vm']
        guesses = [var for var in self._diffeq_names if var in vm_names]
        if len(guesses) == 1: # Unambiguous
            return guesses[0]
        else: # Ambiguous or not found
            return None

    def clean_namespace(self):
        '''
        Removes all variable names from namespaces
        '''
        all_variables = self._eq_names + self._diffeq_names + self._alias.keys() + ['t']
        for name in self._namespace:
            for var in all_variables:
                if var in self._namespace[name]:
                    log_warn('brian.equations', 'Equation variable ' + var + ' also exists in the namespace')
                    del self._namespace[name][var]

    def compile_functions(self, freeze=False):
        """
        Compile all functions defined as strings.
        If freeze is True, all external parameters and units are replaced by their value.
        ALL FUNCTIONS MUST HAVE STRINGS.
        """
        all_variables = self._eq_names + self._diffeq_names + self._alias.keys() + ['t']
        # Check if freezable
        freeze = freeze and all([optimiser.freeze(expr, all_variables, self._namespace[name])\
                               for name, expr in self._string.iteritems()])
        self._frozen = freeze

        # Compile strings to functions
        for name, expr in self._string.iteritems():
            namespace = self._namespace[name] # name space of the function
            # Find variables
            vars = [var for var in get_identifiers(expr) if var in all_variables]
            if freeze:
                expr = optimiser.freeze(expr, all_variables, namespace)
                #self._string[name]=expr # should we?
                #namespace={}
            s = "lambda " + ','.join(vars) + ":" + expr
            self._function[name] = eval(s, namespace)

    def check_units(self):
        '''
        Checks the units of the differential equations, using
        the units of x.
        dx_i/dt must have units of x_i / time.
        '''
        self.set_eq_order()
        # Better: replace xi in the string, or in the namespace
        try:
            for var in self._eq_names:
                f = self._function[var]
                old_func_globals = copy.copy(f.func_globals)
                f.func_globals['xi'] = 0 * second ** -.5 # Noise
                f.func_globals.update(namespace_replace_quantity_with_pure(f.func_globals))
                units = namespace_replace_quantity_with_pure(self._units)
                self.apply(var, units) + self._units[var] # Check that the two terms have the same dimension
                f.func_globals.update(old_func_globals)
            for var in self._diffeq_names:
                f = self._function[var]
                old_func_globals = copy.copy(f.func_globals)
                f.func_globals['xi'] = 0 * second ** -.5 # Noise
                f.func_globals.update(namespace_replace_quantity_with_pure(f.func_globals))
                units = namespace_replace_quantity_with_pure(self._units)
                self.apply(var, units) + (self._units[var] / second) # Check that the two terms have the same dimension
                f.func_globals.update(old_func_globals)
        except DimensionMismatchError, inst:
            raise DimensionMismatchError("The differential equation of " + var + " is not homogeneous", *inst._dims)
        except:
            warnings.warn("Unexpected exception in checking units of " + var)
            raise

    def set_eq_order(self):
        '''
        Computes the internal depency graph of static variables
        and deduces the update order.
        Sets the list of dependencies of dynamic variables on static variables.
        This is called by check_units()
        '''
        if len(self._eq_names) > 0:
            # Internal dependency dictionary
            dependency = {}
            for key in self._eq_names:
                f = self._function[key]
                dependency[key] = [var for var in f.func_code.co_varnames if var in self._eq_names]

            # Sets the order
            staticvars_list = []
            no_dep = None
            while (len(staticvars_list) < len(self._eq_names)) and (no_dep != []):
                no_dep = [key for key, value in dependency.iteritems() if value == []]
                staticvars_list += no_dep
                # Clear dependency list
                for key in no_dep:
                    del dependency[key]
                for key, value in dependency.iteritems():
                    dependency[key] = [var for var in value if not(var in staticvars_list)]

            if no_dep == []: # The dependency graph has cycles!
                raise ReferenceError, "The static variables are referring to each other"
        else:
            staticvars_list = []

        # Calculate dependencies on static variables
        self._dependencies = {}
        for key in staticvars_list:
            self._dependencies[key] = []
            for var in self._function[key].func_code.co_varnames:
                if var in self._eq_names:
                    self._dependencies[key] += [var] + self._dependencies[var]
        for key in self._diffeq_names:
            f = self._function[key]
            self._dependencies[key] = []
            for var in f.func_code.co_varnames:
                if var in self._eq_names:
                    self._dependencies[key] += [var] + self._dependencies[var]

        # Sort the dependency lists
        for key in self._dependencies:
            staticdep = [(staticvars_list.index(var), var) for var in self._dependencies[key]]
            staticdep.sort()
            self._dependencies[key] = [x[1] for x in staticdep]

        # Update _eq
        self._eq_names = staticvars_list

    def substitute_eq(self, name=None):
        """
        Replaces the static variable 'name' by its value in differential
        equations.
        If None: substitute all static variables.
        """
        if name is None:
            for var in self._eq_names[-1::-1]: # reverse order
                self.substitute_eq(var)
        else:
            self.add_prefix_namespace(name)
            #print name
            for var in self._diffeq_names_nonzero:
                # String
                self._string[var] = re.sub("\\b" + name + "\\b", '(' + self._string[name] + ')', self._string[var])
                # Namespace
                self._namespace[var].update(self._namespace[name])
            #print self

    def add_prefix_namespace(self, name):
        """
        Make the variables in the namespace associated to variable name
        specific to that variable by inserting the prefix name_.
        """
        vars = self._namespace[name].keys()
        untransformed_funcs = set(getattr(unitsafefunctions, v) for v in unitsafefunctions.quantity_versions)
        untransformed_funcs.update(set([numpy.clip]))
        for var in vars:
            v = self._namespace[name][var]
            addprefix = True
            if isinstance(v, numpy.ufunc):
                addprefix = False
            try:
                if v in untransformed_funcs:
                    addprefix = False
            except TypeError: #unhashable types
                pass
            if addprefix:
                # String
                self._string[name] = re.sub("\\b" + var + "\\b", name + '_' + var, self._string[name])
                # Namespace
                self._namespace[name][name + '_' + var] = self._namespace[name][var]
                del self._namespace[name][var]

    def free_variables(self):
        """
        Returns the list of free variables (i.e., which are not defined within the
        equation string).
        """
        all_variables = self._eq_names + self._diffeq_names + self._alias.keys() + ['t']
        free_vars = []
        for expr in self._string.itervalues():
            free_vars += [name for name in get_identifiers(expr) if name not in all_variables]
        return list(set(free_vars))

    """
    -----------------------------------------------------------------------
    CALCULATING RHS OF DIFF EQUATIONS
    -----------------------------------------------------------------------
    """
    def apply(self, state, vardict):
        '''
        Calculates self._function[state] with arguments in vardict and
        static variables. The dictionary is filled with the required
        static variables.
        '''
        f = self._function[state]
        # Calculate static variables
        for var in self._dependencies[state]:
            # could add something like: if var not in vardict: this would allow you to override the dependencies if you wanted to - worth doing?
            vardict[var] = call_with_dict(self._function[var], vardict)
        return f(*[vardict[var] for var in f.func_code.co_varnames])

    """
    -----------------------------------------------------------------------
    EQUATION INSPECTION
    -----------------------------------------------------------------------
    """
    def is_stochastic(self, var=None):
        '''
        Returns True if the equation for var is stochastic,
        or if all equations are stochastic (var=None).
        '''
        if var:
            return 'xi' in get_identifiers(self._string[var])
        else:
            return any([self.is_stochastic(name) for name in self._diffeq_names_nonzero])

    def is_time_dependent(self, var=None):
        '''
        Returns True if the equation for var is time dependent,
        or if all equations are time dependent (var=None).
        '''
        if var:
            return 't' in get_identifiers(self._string[var])
        else:
            return any([self.is_time_dependent(name) for name in self._diffeq_names_nonzero])
#            return any([self.is_time_dependent(name) for name in self._diffeq_names_nonzero+self._eq_names])

    def is_linear(self):
        '''
        Returns True if all equations are linear.
        If the equations are time dependent, then returns False.
        If the equations depend on external functions, then returns False.
        '''
        if self.is_time_dependent():
            return False

        for f in self._namespace.iterkeys():
            if any([type(key) == types.FunctionType for key in self._namespace[f].itervalues()]):
                return False
        return all([is_affine(f) for f in self._function.itervalues()])

    def is_conditionally_linear(self):
        '''
        Returns True if the differential equations are linear with respect to the
        state variable.
        '''
        # Equations have to be prepared for it to work.
        for var in self._diffeq_names:
            S = self._units.copy()
            S[var] = AffineFunction()
            try:
                self.apply(var, S)
            except:
                return False
        return True

    """
    -----------------------------------------------------------------------
    NUMERICAL INTEGRATION (to be replaced by code generation)
    -----------------------------------------------------------------------
    """

    def forward_euler(self, S, dt):
        '''
        Updates the value of the state variables in dictionary S
        with the forward Euler algorithm over step dt.
        '''
        # Calculate all static variables (or do that after?)
        #for var in self._eq_names:
        #    S[var]=call_with_dict(self._function[var],S)
        # Calculate derivatives
        buffer = {}
        for varname in self._diffeq_names_nonzero:
            f = self._function[varname]
            buffer[varname] = f(*[S[var] for var in f.func_code.co_varnames])
        # Update variables
        for var in self._diffeq_names_nonzero:
            S[var] += dt * buffer[var]

    def forward_euler_code_string(self):
        '''
        Generates Python code for a forward Euler step.
        '''
        # TODO: check if it can really be frozen
        # TODO: change /a to *(1/a) with precalculation (use parser)
        all_variables = self._eq_names + self._diffeq_names + self._alias.keys() + ['t']
        # nonzero? insert dt?
        vars_tmp = [name + '__tmp' for name in self._diffeq_names]
        lines = ','.join(self._diffeq_names) + '=P._S\n'
        lines += ','.join(vars_tmp) + '=P._dS\n'
        for name in self._diffeq_names_nonzero:
            namespace = self._namespace[name]
            expr = optimiser.freeze(self._string[name], all_variables, namespace)
            lines += name + '__tmp[:]=' + expr + '\n'
        lines += 'P._S+=dt*P._dS\n'
        #print lines
        return lines
        # Return a function f(P) or a namespace (exec code in namespace)
        # 1st option: include directly in neurongroup._state_updater (good?)        

    def forward_euler_code(self):
        '''
        Generates Python code for a forward Euler step.
        '''
        # TODO: check if it can really be frozen
        # TODO: change /a to *(1/a) with precalculation (use parser)
        all_variables = self._eq_names + self._diffeq_names + self._alias.keys() + ['t']
        # nonzero? insert dt?
        vars_tmp = [name + '__tmp' for name in self._diffeq_names]
        lines = ','.join(self._diffeq_names) + '=P._S\n'
        lines += ','.join(vars_tmp) + '=P._dS\n'
        for name in self._diffeq_names_nonzero:
            namespace = self._namespace[name]
            expr = optimiser.freeze(self._string[name], all_variables, namespace)
            lines += name + '__tmp[:]=' + expr + '\n'
        lines += 'P._S+=dt*P._dS\n'
        #print lines
        return compile(lines, 'Euler update code', 'exec')
        # Return a function f(P) or a namespace (exec code in namespace)
        # 1st option: include directly in neurongroup._state_updater (good?)

    def Runge_Kutta2(self, S, dt):
        '''
        Updates the value of the state variables in dictionary S
        with the 2nd order Runge-Kutta algorithm over step dt (midpoint).
        '''
        # Calculate all static variables (or do that after?)
        #for var in self._eq_names:
        #    S[var]=call_with_dict(self._function[var],S)
        # Calculate derivatives
        buffer = {}
        S_half = S.copy()
        # Half a step
        for varname in self._diffeq_names_nonzero:
            f = self._function[varname]
            buffer[varname] = f(*[S[var] for var in f.func_code.co_varnames])
        # Update variables
        for var in self._diffeq_names_nonzero:
            S_half[var] = S[var] + .5 * dt * buffer[var]

        # Whole step
        for varname in self._diffeq_names_nonzero:
            f = self._function[varname]
            buffer[varname] = f(*[S_half[var] for var in f.func_code.co_varnames])
        # Update variables
        for var in self._diffeq_names_nonzero:
            S[var] += dt * buffer[var]

    def exponential_euler(self, S, dt):
        '''
        Updates the value of the state variables in dictionary S
        with an exponential Euler algorithm over step dt.
        Test with is_conditionally_linear first.
        Same as default integration method in Genesis.
        Close to the implicit Euler method in Neuron.
        '''
        # Calculate all static variables (BAD: INSERT IT BELOW)
        #for var in self._eq_names:
        #    S[var]=call_with_dict(self._function[var],S)
        n = len(S[self._diffeq_names_nonzero[0]])
        # Calculate the coefficients of the affine function
        Z = zeros(n)
        O = ones(n)
        A = {}
        B = {}
        for varname in self._diffeq_names_nonzero:
            f = self._function[varname]
            oldval = S[varname]
            S[varname] = Z
            B[varname] = f(*[S[var] for var in f.func_code.co_varnames]).copy() # important if compiled
            S[varname] = O
            A[varname] = f(*[S[var] for var in f.func_code.co_varnames]) - B[varname]
            B[varname] /= A[varname]
            S[varname] = oldval
        # Integrate
        for varname in self._diffeq_names_nonzero:
            f = self._function[varname]
            if self._useweave:
                Bx = B[varname]
                Ax = A[varname]
                Sx = S[varname]
                # Compilation with blitz: we need an approximation because exp is not understood
                #weave.blitz('Sx[:]=-Bx+(Sx+Bx)*(1.+Ax*dt*(1.+.5*Ax*dt))',check_size=0)
                code = """
                for(int k=0;k<n;k++)
                    Sx(k)=-Bx(k)+(Sx(k)+Bx(k))*exp(Ax(k)*dt);
                """
                weave.inline(code, ['n', 'Bx', 'Sx', 'Ax', 'dt'], \
                             compiler=self._cpp_compiler,
                             type_converters=weave.converters.blitz,
                             extra_compile_args=self._extra_compile_args)
            else:
                #S[varname][:]=-B[varname]+(S[varname]+B[varname])*exp(A[varname]*dt)
                # A little faster:
                S[varname] += B[varname]
                S[varname] *= exp(A[varname] * dt)
                S[varname] -= B[varname]

    def exponential_euler_code(self):
        '''
        Generates Python code for an exponential Euler step.
        Not efficient for the moment!
        '''
        all_variables = self._eq_names + self._diffeq_names + self._alias.keys() + ['t']
        vars_tmp = [name + '__tmp' for name in self._diffeq_names]
        lines = ','.join(self._diffeq_names) + '=P._S\n'
        lines += ','.join(vars_tmp) + '=P._dS\n'
        for name in self._diffeq_names:
            # Freeze
            namespace = self._namespace[name]
            expr = optimiser.freeze(self._string[name], all_variables, namespace)
            # Find a and b in dx/dt=a*x+b
            sym_expr = symbolic_eval(expr)
            if isinstance(sym_expr, float):
                lines += name + '__tmp[:]=' + name + '+(' + str(expr) + ')*dt\n'
            else:
                sym_expr = sym_expr.expand()
                sname = sympy.Symbol(name)
                terms = sympy.collect(sym_expr, name, evaluate=False)
                if sname ** 0 in terms:
                    b = terms[sname ** 0]
                else:
                    b = 0
                if sname in terms:
                    a = terms[sname]
                else:
                    a = 0
                lines += name + '__tmp[:]=' + str(-b / a + (sname + b / a) * sympy.exp(a * sympy.Symbol('dt'))) + '\n'
        lines += 'P._S[:]=P._dS'
        #print lines
        return compile(lines, 'Exponential Euler update code', 'exec')

    """
    -------------------
    COMBINING EQUATIONS
    -------------------
    """
    def __add__(self, other):
        '''
        Union of two sets of equations
        '''
        if not isinstance(other, Equations):
            other = Equations(other, level=1)
        result = self.__class__()
        result += self
        result += other
        return result
    __radd__ = __add__

    def __iadd__(self, other):
        if not isinstance(other, Equations):
            other = Equations(other, level=1)
        self._eq_names = list(set(self._eq_names + other._eq_names)) # what to do if same variables?
        self._diffeq_names = list(set(self._diffeq_names + other._diffeq_names))
        self._diffeq_names_nonzero = list(set(self._diffeq_names_nonzero + other._diffeq_names_nonzero))
        self._function = disjoint_dict_union(self._function, other._function)
        self._alias = disjoint_dict_union(self._alias, other._alias)
        self._string = disjoint_dict_union(self._string, other._string)
        self._namespace = disjoint_dict_union(self._namespace, other._namespace)
        # We do this to fix a bug where if you add two Equations together and
        # then create groups from them, the add_prefix_namespace step creates
        # names which can't be correctly resolved in the second NeuronGroup
        # created. This happens because although self._namespace is a new object,
        # self._namespace[var] is shared between the two objects.
        for var in self._namespace.keys():
            self._namespace[var] = copy.copy(self._namespace[var])
        try:
            self._units = disjoint_dict_union(self._units, other._units)
        except AttributeError:
            raise DimensionMismatchError("The two sets of equations do not have compatible units")
        return self

    """
    ---------------------------------
    OTHER METHODS (CALLED EXTERNALLY)
    ---------------------------------
    """
    def fixed_point(self, **kwd):
        '''
        Returns a fixed point of the differential equations
        as a dictionary. The keyword arguments give the (optional)
        initial point (default = 0).
        '''
        values = {}
        for name, value in self._units.iteritems():
            values[name] = 0 * value
        values.update(kwd)
        # Initial vector
        x0 = [values[name] for name in self._diffeq_names_nonzero]
        # Vector function
        def f(x):
            # Put the units back
            x = [xi * get_unit(x0i) for xi, x0i in zip(x, x0)]
            values.update(zip(self._diffeq_names_nonzero, x))
            return [self.apply(name, values) for name in self._diffeq_names_nonzero]
        xf, _, ier, _ = optimize.fsolve(f, x0, full_output=True)
        if ier:
            # Put the units back
            xf = [xfi * get_unit(x0i) for xfi, x0i in zip(xf, x0)]
            # Return a dictionary
            return dict(zip(self._diffeq_names_nonzero, xf))
        else: # Not found
            warnings.warn('Could not find a fixed point of the equations')
            return kwd

    def substitute(self, name1, name2):
        """
        Changes name1 to name2 (variable names).
        Note: I don't where this is called!
        """
        # Aliases
        if name1 in self._alias:
            self._alias[name2] = self._alias[name1]
            del self._alias[name1]
        # Units
        if name1 in self._units:
            self._units[name2] = self._units[name1]
            del self._units[name1]
        # Equations
        if name1 in self._eq_names:
            self._eq_names[self._eq_names.index(name1)] = name2
        # Differential equations
        if name1 in self._diffeq_names:
            self._diffeq_names[self._diffeq_names.index(name1)] = name2
        if name1 in self._diffeq_names_nonzero:
            self._diffeq_names_nonzero[self._diffeq_names_nonzero.index(name1)] = name2
        # Strings
        if name1 in self._string:
            self._string[name2] = self._string[name1]
            del self._string[name1]
        for name, value in self._string.iteritems():
            self._string[name] = re.sub("\\b" + name1 + "\\b", name2, value)
        # Namespaces
        if name1 in self._namespace:
            self._namespace[name2] = self._namespace[name1]
            del self._namespace[name1]

    def __getattr__(self, name):
        '''
        Returns the corresponding function.
        '''
        # bug with clustertools
        if name == 'as_array':
            raise Exception()
        return lambda ** kwd:self.apply(name, kwd)

    def __len__(self):
        '''
        Number of differential equations

        Note: is this still used?
        '''
        return len(self._diffeq_names)

    def __repr__(self):
        s = ''
        for var in self._diffeq_names:
            s += 'd' + var + '/dt = ' + self._string[var] + ' [diffeq]\n'
        for var in self._eq_names:
            if var in self._alias:
                typename = ' [alias]'
            else:
                typename = ' [eq]'
            s += var + ' = ' + self._string[var] + typename + '\n'
        return s

    def __reduce__(self):
        # To avoid recursion, we temporarily set the class to a trivial
        # class, this is restored at the end, and by _load_Equations_from_pickle
        # too
        self.__class__, cls = PickledEquations, self.__class__
        selfcopy = copy.copy(self)
        # We need to delete __builtins__ from all the namespaces as it is
        # not picklable, so we make copies
        selfcopy._namespace = copy.copy(selfcopy._namespace)
        for key in selfcopy._namespace.keys():
            selfcopy._namespace[key] = copy.copy(selfcopy._namespace[key])
            if '__builtins__' in selfcopy._namespace[key]:
                del selfcopy._namespace[key]['__builtins__']
        selfcopy._function = {}
        # Sometimes namespaces have numpy ufuncs in them, which are not
        # picklable, so we replace them with PickledUfunc objects which just
        # store their name, and _load_equations_from_pickle will extract them
        # from numpy again
        def replaceufunc(d):
            for k in d.keys():
                v = d[k]
                if isinstance(v, numpy.ufunc):
                    d[k] = PickledUfunc(v)
                if isinstance(v, dict):
                    replaceufunc(v)
        replaceufunc(selfcopy.__dict__)
        self.__class__ = cls
        return (_load_Equations_from_pickle, (selfcopy, cls))

class PickledUfunc(object):
    def __init__(self, ufunc):
        self.name = ufunc.__name__
    def get(self):
        return getattr(numpy, self.name)

class PickledEquations(object):
    pass

def _load_Equations_from_pickle(eqs, cls):
    def replaceufunc(d):
        for k in d.keys():
            v = d[k]
            if isinstance(v, PickledUfunc):
                d[k] = v.get()
            if isinstance(v, dict):
                replaceufunc(v)
    replaceufunc(eqs.__dict__)
    eqs.__class__ = cls
    eqs.prepare()
    return eqs

# Utilitary functions
# -------------------
def call_with_dict(f, d):
    '''
    Calls a function f with arguments from dictionary d.
    The dictionary can contain keys that are not variables of f.
    '''
    return f(*[d[var] for var in f.func_code.co_varnames])

def disjoint_dict_union(d1, d2):
    '''
    Merges the dictionaries d1 and d2 and checks that
    they are compatible (i.e., raises an exception if d1[key]!=d2[key])
    '''
    result = {}
    result.update(d1)
    for key, value in d2.iteritems(): # Bug here
        if (key in d1) and (d1[key] != value):
            raise AttributeError, "Incompatible dictionaries in disjoint union, problem with key " + key
        result[key] = value
    return result



'''
Chaos in the AdEx model
-----------------------
Fig. 8B from:
Touboul, J. and Brette, R. (2008). Dynamics and bifurcations of the adaptive
exponential integrate-and-fire model. Biological Cybernetics 99(4-5):319-34.

This shows the bifurcation structure when the reset value is varied
(vertical axis shows the values of w at spike times for a given a reset value
Vr).
'''
from brian import *

defaultclock.dt=0.01*ms

C=281*pF
gL=30*nS
EL=-70.6*mV
VT=-50.4*mV
DeltaT=2*mV
tauw=40*ms
a=4*nS
b=0.08*nA
I=.8*nA
Vcut=VT+5*DeltaT # practical threshold condition
N=500

eqs="""
dvm/dt=(gL*(EL-vm)+gL*DeltaT*exp((vm-VT)/DeltaT)+I-w)/C : volt
dw/dt=(a*(vm-EL)-w)/tauw : amp
Vr:volt
"""

neuron=NeuronGroup(N,model=eqs,threshold=Vcut,reset="vm=Vr;w+=b")
neuron.vm=EL
neuron.w=a*(neuron.vm-EL)
neuron.Vr=linspace(-48.3*mV,-47.7*mV,N) # bifurcation parameter

run(3*second,report='text') # we discard the first spikes

M=StateSpikeMonitor(neuron,("Vr","w")) # record Vr and w at spike times
run(2*second,report='text')

Vr,w=M.values("Vr"),M.values("w")

figure()
plot(Vr/mV,w/nA,'.k')
xlabel('Vr (mV)')
ylabel('w (nA)')
show()



"""
Module to deal with the AER (Address Event Representation) format.

Current state:
* load_AER seems fine
* extract_DVS_event is probably fine too, but maybe should be in a "chip" module?

"""

from numpy import *
from brian.directcontrol import SpikeGeneratorGroup
from brian.units import *
from brian.neurongroup import *
from brian.directcontrol import SpikeGeneratorThreshold
from brian.monitor import SpikeMonitor, FileSpikeMonitor
from brian.clock import guess_clock
from brian.stateupdater import *

import os, datetime, struct
__all__=['load_AER','save_AER',
         'extract_DVS_event', 'extract_AMS_event',
         'AERSpikeGeneratorGroup', 'AERSpikeMonitor']

################# Fast Direct Control ###############################
# TODO: this could probably be moved to the main directcontrol module

class AERSpikeGeneratorGroup(NeuronGroup):
    '''
    This class loads AER data files and puts them in a SpikeGeneratorGroup for use in Brian.
    (one can find sample data files in http://sourceforge.net/apps/trac/jaer/wiki/AER%20data)
    
    This can load any AER files that is supported by load_AER, apart from index (.aeidx) files that point to multiple data files. Check the documentation for load_AER for that.
    
    Sample usage:
    Gin = AERSpikeGeneratorGroup('/path/to/file/samplefile.dat')
    or
    Gin = AERSpikeGeneratorGroup((addr,timestamps))
    or
    Gin = AERSpikeGeneratorGroup(pickled_spike_monitor)

    Attributes:
    maxtime : is the timing of the last spike of the object
    '''
    def __init__(self, data, clock = None, timeunit = 1*usecond, relative_time = True):
        if isinstance(data, str):
            l = data.split('.')
            ext = l[-1].strip('\n')
            if ext == 'aeidx':
                raise ValueError('Cannot create a single AERSpikeGeneratorGroup with aeidx files. Consider using load_AER first and manually create multiple AERSpikeGeneratorGroups.')
            else:
                data = load_AER(data, relative_time = relative_time, check_sorted = True)
        if isinstance(data, SpikeMonitor):
            addr, time = zip(*data.spikes)
            addr = np.array(list(addr))
            timestamps = np.array(list(time))
        elif isinstance(data, tuple):
            addr, timestamps = data
            
        self.tmax = max(timestamps)*timeunit
        self._nspikes = len(addr)
        N = max(addr) + 1
        clock = guess_clock(clock)
        threshold = FastDCThreshold(addr, timestamps*timeunit, dt = clock.dt)
        NeuronGroup.__init__(self, N, model = LazyStateUpdater(), threshold = threshold, clock = clock)
    
    @property
    def maxtime(self):
        # this should be kept for AER generated groups, because then one can use run(group.maxtime)
        if not isinstance(self.tmax, Quantity):
            return self.tmax*second
        return self.tmax
    
    @property
    def nspikes(self):
        return self._nspikes
        
class FastDCThreshold(SpikeGeneratorThreshold):
    '''
    Implementing dan's idea for fast Direct Control Threshold, works like a charm.
    '''
    def __init__(self, addr, timestamps, dt = None):
        self.set_offsets(addr, timestamps, dt = dt)
        
    def set_offsets(self, I, T, dt = 1000):
        # Convert times into integers
        T = array(T/dt, dtype=int)
        # Put them into order
        # We use a field array to sort first by time and then by neuron index
        spikes = zeros(len(I), dtype=[('t', int), ('i', int)])
        spikes['t'] = T
        spikes['i'] = I
        spikes.sort(order=('t', 'i'))
        T = spikes['t']
        self.I = spikes['i']
        # Now for each timestep, we find the corresponding segment of I with
        # the spike indices for that timestep.
        # The idea of offsets is that the segment offsets[t]:offsets[t+1]
        # should give the spikes with time t, i.e. T[offsets[t]:offsets[t+1]]
        # should all be equal to t, and so then later we can return
        # I[offsets[t]:offsets[t+1]] at time t. It might take a bit of thinking
        # to see why this works. Since T is sorted, and bincount[i] returns the
        # number of elements of T equal to i, then j=cumsum(bincount(T))[t]
        # gives the first index in T where T[j]=t.
        self.offsets = hstack((0, cumsum(bincount(T))))
    
    def __call__(self, P):
        t = P.clock.t
        dt = P.clock.dt
        t = int(round(t/dt))
        if t+1>=len(self.offsets):
            return array([], dtype=int)
        return self.I[self.offsets[t]:self.offsets[t+1]]

########### AER loading stuff ######################

def load_multiple_AER(filename, check_sorted = False, relative_time = False, directory = '.'):
    f=open(filename,'rb')
    line = f.readline()
    res = []
    line = line.strip('\n')
    while not line == '':
        res.append(load_AER(os.path.join(directory, line), check_sorted = check_sorted, relative_time = relative_time))
        line = f.readline()
    f.close()
    return res

def load_AER(filename, check_sorted = False, relative_time = True):
    '''
    Loads AER data files for use in Brian.
    Returns a list containing tuples with a vector of addresses and a vector of timestamps (ints, unit is usually usecond).

    It can load any kind of .dat, or .aedat files.
    Note: For index files (that point to multiple .(ae)dat files) it will return a list containing tuples as for single files.
    
    Keyword Arguments:
    If check_sorted is True, checks if timestamps are sorted,
    and sort them if necessary.
    If relative_time is True, it will set the first spike time to zero and all others relatively to that precise time (avoid negative timestamps, is definitely a good idea).
    
    Hence to use those data files in Brian, one should do:

    addr, timestamp =  load_AER(filename, relative_time = True)
    G = AERSpikeGeneratorGroup((addr, timestamps))
    '''
    l = filename.split('.')
    ext = l[-1].strip('\n')
    filename = filename.strip('\n')
    directory = os.path.dirname(filename)
    if ext == 'aeidx':
        #AER data points to different AER files
        return load_multiple_AER(filename, check_sorted = check_sorted, relative_time = relative_time, directory = directory)
    elif not (ext == 'dat' or ext == 'aedat'):
        raise ValueError('Wrong extension for AER data, should be dat, or aedat, it was '+ext)
    
    # This is inspired by the following Matlab script:
    # http://jaer.svn.sourceforge.net/viewvc/jaer/trunk/host/matlab/loadaerdat.m?revision=2001&content-type=text%2Fplain
    f=open(filename,'rb')
    version=1 # default (if not found in the file)
    
    # Skip header and look for version number
    line = f.readline()
    while line[0] == '#':
        if line[:9] == "#!AER-DAT":
            version = int(float(line[9:-1]))
        line = f.readline()
    line += f.read()
    f.close()
    
    if version==1:
        print 'Loading version 1 file '+filename
        '''
        Format is: sequence of (addr = 2 bytes,timestamp = 4 bytes)
        Number format is big endian ('>')
        '''
        ## This commented paragraph is the non-vectorized version
        #nevents=len(line)/6
        #for n in range(nevents):
        #    events.append(unpack('>HI',line[n*6:(n+1)*6])) # address,timestamp
        x=fromstring(line, dtype=int16) # or uint16?
        x=x.reshape((len(x)/3,3))
        addr=x[:,0].newbyteorder('>')
        timestamp=x[:,1:].copy()
        timestamp.dtype=int32
        timestamp=timestamp.newbyteorder('>').flatten()
    else: # version==2
        print 'Loading version 2 file '+filename
        '''
        Format is: sequence of (addr = 4 bytes,timestamp = 4 bytes)
        Number format is big endian ('>')
        '''
        ## This commented paragraph is the non-vectorized version
        #nevents=len(line)/8
        #for n in range(nevents):
        #    events.append(unpack('>II',line[n*8:(n+1)*8])) # address,timestamp
        x = fromstring(line, dtype=int32).newbyteorder('>')
        addr = x[::2]
        if len(addr) == len(x[1::2]):
            timestamp = x[1::2]
        else:
            print """It seems there was a problem with the AER file, timestamps and addr don't have the same length!"""
            timestamp = x[1::2]

    if check_sorted: # Sorts the events if necessary
        if any(diff(timestamp)<0): # not sorted
            ind = argsort(timestamp)
            addr,timestamp = addr[ind],timestamp[ind]
    if (timestamp<0).all():
        print 'Negative timestamps'
    
    if relative_time:
        t0 = min(timestamp)
        timestamp -= t0
    
    return addr,timestamp

HEADER = """#!AER-DAT2.0\n# This is a raw AE data file - do not edit\n# Data format is int32 address, int32 timestamp (8 bytes total), repeated for each event\n# Timestamps tick is 1 us\n# created with the Brian simulator on """

def save_AER(spikemonitor, f):
    '''
    Saves the SpikeMonitor's contents to a file in aedat format.
    File should have 'aedat' extension.
    One can specify an open file, or, alternatively the filename as a string.

    Usage:
    save_AER(spikemonitor, file)
    '''
    if isinstance(spikemonitor, SpikeMonitor):
        spikes = spikemonitor.spikes
    else:
        spikes = spikemonitor
    if isinstance(f, str):
        strinput = True
        f = open(f, 'wb')
    l = f.name.split('.')
    if not l[-1] == 'aedat':
        raise ValueError('File should have aedat extension')
    header = HEADER
    header += str(datetime.datetime.now()) + '\n'
    f.write(header)
    # i,t=zip(*spikes)
    for (i,t) in spikes:
        addr = struct.pack('>i', i)
        f.write(addr)
        time = struct.pack('>i', int(ceil(float(t/usecond))))
        f.write(time)
    if strinput:
        f.close()
    
class AERSpikeMonitor(FileSpikeMonitor):
    """Records spikes to an AER file
    
    Initialised as::
    
        FileSpikeMonitor(source, filename[, record=False])
    
    Does everything that a :class:`SpikeMonitor` does except ONLY records
    the spikes to the named file in AER format. 

    
    Has one additional method:
    
    ``close_file()``
        Closes the file manually (will happen automatically when
        the program ends).
    """
    def __init__(self, source, filename, record=False, delay=0):
        super(FileSpikeMonitor, self).__init__(source, record, delay)
        self.filename = filename
        self.f = open(filename, 'w')
        header = HEADER
        header += str(datetime.datetime.now()) + '\n'
        self.f.write(header)

    def propagate(self, spikes):
#        super(AERSpikeMonitor, self).propagate(spikes)
        # TODO do it better, no struct.pack! check numpy doc for 
        
#        addr = array(spikes).newbyteorder('>')
        for i in spikes:
            addr = struct.pack('>i', i)
            self.f.write(addr)
            time = struct.pack('>i', int(ceil(float(self.source.clock.t/usecond))))
            self.f.write(time)
    
########### AER addressing stuff ######################

def extract_DVS_event(addr):
    '''
    Extracts retina event from an address or a vector of addresses.
    
    Chip: Digital Vision Sensor (DVS)
    http://siliconretina.ini.uzh.ch/wiki/index.php
    
    Returns: x, y, polarity (ON/OFF: 1/-1)
    '''
    retina_size=128

    xmask = 0xfE # x are 7 bits (64 cols) ranging from bit 1-8
    ymask = 0x7f00 # y are also 7 bits
    xshift=1 # bits to shift x to right
    yshift=8 # bits to shift y to right
    polmask=1 # polarity bit is LSB

    x = retina_size - 1 - ((addr & xmask) >> xshift)
    y = (addr & ymask) >> yshift
    pol = 1 - 2*(addr & polmask) # 1 for ON, -1 for OFF
    return x,y,pol

def extract_AMS_event(addr):
    '''
    Extracts cochlea event from an address or a vector of addresses

    Chip: Silicon Cochlea (AMS)
    
    Returns: side, channel, filternature
    
    More precisely:
    side: 0 is left, 1 is right
    channel: apex (LF) is 63, base (HF) is 0
    filternature: 0 is lowpass, 1 is bandpass
    '''
    # Reference:
    # ch.unizh.ini.jaer.chip.cochlea.CochleaAMSNoBiasgen.Extractor in the jAER package (look in the javadoc)
    # also the cochlea directory in jAER/host/matlab has interesting stuff
    # the matlab code was used to write this function. I don't understand the javadoc stuff
    #cochlea_size = 64

    xmask = 31 # x are 5 bits 32 channels) ranging from bit 1-5 
    ymask = 32 # y (one bit) determines left or right cochlea
    xshift=0 # bits to shift x to right
    yshift=5 # bits to shift y to right
    
    channel = 1 + ((addr & xmask) >> xshift)
    side = (addr & ymask) >> yshift
    lpfBpf = mod(addr, 2)
#    leftRight = mod(addr, 4)
    return (lpfBpf, side, channel)

if __name__=='__main__':
    path=r'C:Users\Romain\Desktop\jaerSampleData\DVS128'
    filename=r'\Tmpdiff128-2006-02-03T14-39-45-0800-0 tobi eye.dat'

    addr,timestamp=load_AER(path+filename)



from brian import *

#set_global_preferences(usecodegen=False)

N = 5
freq = linspace(0, 1, 5)

eqs = '''
dV/dt = cos(freq*V)/(1*second) : 1
#dV/dt = freq*(1-V)/(1*second) : 1
#freq : 1
'''

G = NeuronGroup(N, eqs)
#G.freq = freq
M = StateMonitor(G, 'V', record=True)

run(1*second)

M.plot()
show()


from brian import *

gmax = 1.5
tau_pre = 20 * ms
tau_post = tau_pre
dA_pre = .001
dA_post = -dA_pre * tau_pre / tau_post * 1.05

input = PoissonGroup(2, 50 * Hz)
output = NeuronGroup(2, 'V:1',
                     reset=0, threshold=0.5)
relay = IdentityConnection(input, output, delay=1 * ms)
C = Connection(input, output)
C[0, 0] = C[1, 1] = 1e-100

eqs_stdp = '''
dA_pre/dt = -A_pre/tau_pre : 1
dA_post/dt = -A_post/tau_post : 1
'''
dA_post *= gmax
dA_pre *= gmax
class Heap(object):
    pass
h = Heap()
h.dA_post = dA_post
h.dA_pre = dA_pre
stdp = STDP(C , eqs=eqs_stdp, pre='A_pre+=h.dA_pre; w+=A_post',
          post='A_post+=h.dA_post; w+=A_pre', wmax=gmax)

Mi = SpikeMonitor(input)
Mo = SpikeMonitor(output)

w0 = []
w1 = []
@network_operation(clock=EventClock(dt=1 * ms))
def recweights():
    w0.append(C[0, 0])
    w1.append(C[1, 1])

run(1 * second)
h.dA_post *= 5
h.dA_pre *= 5
run(1 * second)
plot(w0)
plot(w1)
show()



from brian import *
from scipy import sparse

P=NeuronGroup(1000,model='v:volt')
C=Connection(P,P,sparseness=0.1,weight=1*mV)
C2=Connection(P,P[0])
for i in range(len(P)):
    #print C[i,:].sum() # but sum(C[i,:]) does not work
    C2[i,0]=C[i,:].sum()

W=sparse.lil_matrix((5,5))
print W[1,:].sum() # same problem for sum(W[1,:])



"""
Implementation of synfire chain from Diesmann et al. 1999

Dan Goodman - Dec. 2007
"""

#import brian_no_units
from brian import *
import time

from brian.library.IF import *
from brian.library.synapses import *

def minimal_example():
    # Neuron model parameters
    Vr = -70 * mV
    Vt = -55 * mV
    taum = 10 * ms
    taupsp = 0.325 * ms
    weight = 4.86 * mV
    # Neuron model
    equations = Equations('''
        dV/dt = (-(V-Vr)+x)*(1./taum)                            : volt
        dx/dt = (-x+y)*(1./taupsp)                               : volt
        dy/dt = -y*(1./taupsp)+25.27*mV/ms+(39.24*mV/ms**0.5)*xi : volt
        ''')

    # Neuron groups
    P = NeuronGroup(N=1000, model=equations,
                  threshold=Vt, reset=Vr, refractory=1 * ms)
#    P = NeuronGroup(N=1000, model=(dV,dx,dy),init=(0*volt,0*volt,0*volt),
#                  threshold=Vt,reset=Vr,refractory=1*ms)

    Pinput = PulsePacket(t=50 * ms, n=85, sigma=1 * ms)
    # The network structure
    Pgp = [ P.subgroup(100) for i in range(10)]
    C = Connection(P, P, 'y')
    for i in range(9):
        C.connect_full(Pgp[i], Pgp[i + 1], weight)
    Cinput = Connection(Pinput, P, 'y')
    Cinput.connect_full(Pinput, Pgp[0], weight)
    # Record the spikes
    Mgp = [SpikeMonitor(p, record=True) for p in Pgp]
    Minput = SpikeMonitor(Pinput, record=True)
    monitors = [Minput] + Mgp
    # Setup the network, and run it
    P.V = Vr + rand(len(P)) * (Vt - Vr)
    run(100 * ms)
    # Plot result
    raster_plot(showgrouplines=True, *monitors)
    show()


# DEFAULT PARAMATERS FOR SYNFIRE CHAIN
# Approximates those in Diesman et al. 1999
model_params = Parameters(
    # Simulation parameters
    dt=0.1 * ms,
    duration=100 * ms,
    # Neuron model parameters
    taum=10 * ms,
    taupsp=0.325 * ms,
    Vt= -55 * mV,
    Vr= -70 * mV,
    abs_refrac=1 * ms,
    we=34.7143,
    wi= -34.7143,
    psp_peak=0.14 * mV,
    # Noise parameters
    noise_neurons=20000,
    noise_exc=0.88,
    noise_inh=0.12,
    noise_exc_rate=2 * Hz,
    noise_inh_rate=12.5 * Hz,
    computed_model_parameters="""
    noise_mu = noise_neurons * (noise_exc * noise_exc_rate - noise_inh * noise_inh_rate ) * psp_peak * we
    noise_sigma = (noise_neurons * (noise_exc * noise_exc_rate + noise_inh * noise_inh_rate ))**.5 * psp_peak * we
    """
    )

# MODEL FOR SYNFIRE CHAIN
# Excitatory PSPs only
def Model(p):
    equations = Equations('''
        dV/dt = (-(V-p.Vr)+x)*(1./p.taum)                          : volt
        dx/dt = (-x+y)*(1./p.taupsp)                               : volt
        dy/dt = -y*(1./p.taupsp)+25.27*mV/ms+(39.24*mV/ms**0.5)*xi : volt
        ''')
    return Parameters(model=equations, threshold=p.Vt, reset=p.Vr, refractory=p.abs_refrac)

default_params = Parameters(
    # Network parameters
    num_layers=10,
    neurons_per_layer=100,
    neurons_in_input_layer=100,
    # Initiating burst parameters
    initial_burst_t=50 * ms,
    initial_burst_a=85,
    initial_burst_sigma=1 * ms,
    # these values are recomputed whenever another value changes
    computed_network_parameters="""
    total_neurons = neurons_per_layer * num_layers
    """,
    # plus we also use the default model parameters
    ** model_params
    )

# DEFAULT NETWORK STRUCTURE
# Single input layer, multiple chained layers
class DefaultNetwork(Network):
    def __init__(self, p):
        # define groups
        chaingroup = NeuronGroup(N=p.total_neurons, **Model(p))
        inputgroup = PulsePacket(p.initial_burst_t, p.neurons_in_input_layer, p.initial_burst_sigma)
        layer = [ chaingroup.subgroup(p.neurons_per_layer) for i in range(p.num_layers) ]
        # connections
        chainconnect = Connection(chaingroup, chaingroup, 2)
        for i in range(p.num_layers - 1):
            chainconnect.connect_full(layer[i], layer[i + 1], p.psp_peak * p.we)
        inputconnect = Connection(inputgroup, chaingroup, 2)
        inputconnect.connect_full(inputgroup, layer[0], p.psp_peak * p.we)
        # monitors
        chainmon = [SpikeMonitor(g, True) for g in layer]
        inputmon = SpikeMonitor(inputgroup, True)
        mon = [inputmon] + chainmon
        # network
        Network.__init__(self, chaingroup, inputgroup, chainconnect, inputconnect, mon)
        # add additional attributes to self
        self.mon = mon
        self.inputgroup = inputgroup
        self.chaingroup = chaingroup
        self.layer = layer
        self.params = p

    def prepare(self):
        Network.prepare(self)
        self.reinit()

    def reinit(self, p=None):
        Network.reinit(self)
        q = self.params
        if p is None: p = q
        self.inputgroup.generate(p.initial_burst_t, p.initial_burst_a, p.initial_burst_sigma)
        self.chaingroup.V = q.Vr + rand(len(self.chaingroup)) * (q.Vt - q.Vr)

    def run(self):
        Network.run(self, self.params.duration)

    def plot(self):
        raster_plot(ylabel="Layer", title="Synfire chain raster plot",
                   color=(1, 0, 0), markersize=3,
                   showgrouplines=True, spacebetweengroups=0.2, grouplinecol=(0.5, 0.5, 0.5),
                   *self.mon)

def estimate_params(mon, time_est):
    # Quick and dirty algorithm for the moment, for a more decent algorithm
    # use leastsq algorithm from scipy.optimize.minpack to fit const+Gaussian
    # http://www.scipy.org/doc/api_docs/SciPy.optimize.minpack.html#leastsq
    i, times = zip(*mon.spikes)
    times = array(times)
    times = times[abs(times - time_est) < 15 * ms]
    if len(times) == 0:
        return (0, 0 * ms)
    better_time_est = times.mean()
    times = times[abs(times - time_est) < 5 * ms]
    if len(times) == 0:
        return (0, 0 * ms)
    return (len(times), times.std())

def single_sfc():
    net = DefaultNetwork(default_params)
    net.run()
    net.plot()

def state_space(grid, neuron_multiply, verbose=True):
    amin = 0
    amax = 100
    sigmamin = 0. * ms
    sigmamax = 3. * ms

    params = default_params()
    params.num_layers = 1
    params.neurons_per_layer = params.neurons_per_layer * neuron_multiply

    net = DefaultNetwork(params)

    i = 0
    # uncomment these 2 lines for TeX labels
    #import pylab
    #pylab.rc_params.update({'text.usetex': True}) 
    if verbose:
        print "Completed:"
    start_time = time.time()
    figure()
    for ai in range(grid + 1):
        for sigmai in range(grid + 1):
            a = int(amin + (ai * (amax - amin)) / grid)
            if a > amax: a = amax
            sigma = sigmamin + sigmai * (sigmamax - sigmamin) / grid
            params.initial_burst_a, params.initial_burst_sigma = a, sigma
            net.reinit(params)
            net.run()
            (newa, newsigma) = estimate_params(net.mon[-1], params.initial_burst_t)
            newa = float(newa) / float(neuron_multiply)
            col = (float(ai) / float(grid), float(sigmai) / float(grid), 0.5)
            plot([sigma / ms, newsigma / ms], [a, newa], color=col)
            plot([sigma / ms], [a], marker='.', color=col, markersize=15)
            i += 1
            if verbose:
                print str(int(100. * float(i) / float((grid + 1) ** 2))) + "%",
        if verbose:
            print
    if verbose:
        print "Evaluation time:", time.time() - start_time, "seconds"
    xlabel(r'$\sigma$ (ms)')
    ylabel('a')
    title('Synfire chain state space')
    axis([sigmamin / ms, sigmamax / ms, amin, amax])


minimal_example()
#print 'Computing SFC with multiple layers'
#single_sfc()
#print 'Plotting SFC state space'
#state_space(3,1)
#state_space(8,10)
#state_space(10,50)
#state_space(10,150)
#show()



import pycuda.autoinit as autoinit
import pycuda.driver as drv
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
import numpy, pylab, time

N = 1000000
x0 = 3.2
block = (512, 1, 1)
grid = (int(N / 512) + 1, 1)

mod = drv.SourceModule("""
__global__ void threshold(float *x, float x0, int *J, unsigned int *global_j, int N)
{
 int i = blockIdx.x * blockDim.x + threadIdx.x;
 if(x[i]>x0 && i<N){
  unsigned int j = atomicInc(global_j, N);
  J[j] = i;
 }
}
""")

threshold = mod.get_function("threshold")

v = gpuarray.to_gpu(numpy.array(numpy.random.randn(N), dtype=numpy.float32))

J = drv.mem_alloc(4 * N)
global_j = drv.mem_alloc(4)

Jret = numpy.zeros(N, dtype=int)
jret = numpy.zeros(1, dtype=numpy.uint32)

drv.memcpy_htod(J, numpy.zeros(N, dtype=int))
drv.memcpy_htod(global_j, numpy.zeros(1, dtype=numpy.uint32))

start = time.time()
threshold(v, numpy.float32(x0), J, global_j, numpy.int32(N),
            block=block, grid=grid)
drv.memcpy_dtoh(jret, global_j)
Jret = Jret[:jret[0]]
drv.memcpy_dtoh(Jret, J)
print 'GPU time without sorting:', time.time() - start
#Jret = Jret[:jret[0]]
Jret.sort()
print 'GPU time with sorting on CPU:', time.time() - start

#print Jret#[Jret<N]

start = time.time()
Jcpu = numpy.where(v.get() > x0)[0]
print'CPU time with numpy:', time.time() - start

print all(Jret == Jcpu)
print len(Jret), float(len(Jret)) / N



from brian import *
from filterbank import Filterbank, FunctionFilterbank, CombinedFilterbank
from filterbanklibrary import *

__all__ = ['DRNL']

def set_parameters(cf,type,param):
    
    parameters=dict()
    parameters['stape_scale']=0.00014 
    parameters['order_linear']=3
    parameters['order_nonlinear']=3

    if type=='guinea pig':
        parameters['cf_lin_p0']=0.339
        parameters['cf_lin_m']=0.339
        parameters['bw_lin_p0']=1.3
        parameters['bw_lin_m']=0.53
        parameters['cf_nl_p0']=0
        parameters['cf_nl_m']=1.
        parameters['bw_nl_p0']=0.8
        parameters['bw_nl_m']=0.58
        parameters['a_p0']=1.87
        parameters['a_m']=0.45
        parameters['b_p0']=-5.65
        parameters['b_m']=0.875
        parameters['c_p0']=-1.
        parameters['c_m']=0
        parameters['g_p0']=5.68
        parameters['g_m']=-0.97
        parameters['lp_lin_cutoff_p0']=0.339
        parameters['lp_lin_cutoff_m']=0.339
        parameters['lp_nl_cutoff_p0']=0
        parameters['lp_nl_cutoff_m']=1.
        
    elif type=='human':
        parameters['cf_lin_p0']=-0.067
        parameters['cf_lin_m']=1.016
        parameters['bw_lin_p0']=0.037
        parameters['bw_lin_m']=0.785
        parameters['cf_nl_p0']=-0.052
        parameters['cf_nl_m']=1.016
        parameters['bw_nl_p0']=-0.031
        parameters['bw_nl_m']=0.774
        parameters['a_p0']=1.402
        parameters['a_m']=0.819
        parameters['b_p0']=1.619
        parameters['b_m']=-0.818  
        parameters['c_p0']=-0.602
        parameters['c_m']=0
        parameters['g_p0']=4.2
        parameters['g_m']=0.48
        parameters['lp_lin_cutoff_p0']=-0.067
        parameters['lp_lin_cutoff_m']=1.016
        parameters['lp_nl_cutoff_p0']=-0.052
        parameters['lp_nl_cutoff_m']=1.016
        
    if param: 
        if not isinstance(param, dict): 
            raise Error('given parameters must be a dict')
        for key in param.keys():
            if not parameters.has_key(key):
                raise Exception(key + ' is invalid key entry for given parameters')
            parameters[key] = param[key]
    return parameters
    
class DRNL(CombinedFilterbank):
    r'''
    Implementation of the dual resonance nonlinear (DRNL) filter
    as described in Lopez-Paveda, E. and Meddis, R.,
    "A human nonlinear cochlear filterbank", JASA 2001.
    
    The entire pathway consists of the sum of a linear and a nonlinear pathway.
    
    The linear path consists of a bank of bandpass filters (second order
    gammatone), a low pass function, and a gain/attenuation factor, g, in a
    cascade.
    
    The nonlinear path is a cascade consisting of a bank of gammatone filters, a
    compression function, a second bank of gammatone filters, and a low
    pass function, in that order.

    Initialised with arguments:
    
    ``source``
        Source of the cochlear model.
        
    ``cf``
        List or array of center frequencies.
        
    ``type``
        defines the parameters set corresponding to a certain fit. It can be
        either:
        
        ``type='human'`` 
            The parameters come from Lopez-Paveda, E. and Meddis, R.., "A human
            nonlinear cochlear filterbank", JASA 2001.
        
        ``type ='guinea pig'``
            The parameters come from Summer et al., "A nonlinear filter-bank
            model of the guinea-pig cochlear nerve: Rate responses", JASA 2003.
        
    ``param``
        Dictionary used to overwrite the default parameters given in the
        original papers. 
    
    The possible parameters to change and their default values for humans (see 
    Lopez-Paveda, E. and Meddis, R.,"A human nonlinear cochlear filterbank",
    JASA 2001. for notation) are::
      
      param['stape_scale']=0.00014 
      param['order_linear']=3 
      param['order_nonlinear']=3 
    
    from there on the parameters are given in the form
    :math:`x=10^{\mathrm{p0}+m\log_{10}(\mathrm{cf})}` where
    ``cf`` is the center frequency::
    
      param['cf_lin_p0']=-0.067
      param['cf_lin_m']=1.016
      param['bw_lin_p0']=0.037
      param['bw_lin_m']=0.785
      param['cf_nl_p0']=-0.052
      param['cf_nl_m']=1.016
      param['bw_nl_p0']=-0.031
      param['bw_nl_m']=0.774
      param['a_p0']=1.402
      param['a_m']=0.819
      param['b_p0']=1.619
      param['b_m']=-0.818  
      param['c_p0']=-0.602
      param['c_m']=0
      param['g_p0']=4.2
      param['g_m']=0.48
      param['lp_lin_cutoff_p0']=-0.067
      param['lp_lin_cutoff_m']=1.016
      param['lp_nl_cutoff_p0']=-0.052
      param['lp_nl_cutoff_m']=1.016   
    '''
    
    def __init__(self, source, cf, type='human', param={}):

        CombinedFilterbank.__init__(self, source)
        source = self.get_modified_source()
        
        cf = atleast_1d(cf)
        nbr_cf=len(cf)
        parameters=set_parameters(cf,type,param)
        
        #conversion to stape velocity (which are the units needed for the further centres)
        source=source*parameters['stape_scale'] 
        
        #### Linear Pathway ####
        #bandpass filter (second order  gammatone filter)
        cf_linear=10**(parameters['cf_lin_p0']+parameters['cf_lin_m']*log10(cf))
        bandwidth_linear=10**(parameters['bw_lin_p0']+parameters['bw_lin_m']*log10(cf))
        gammatone=ApproximateGammatone(source, cf_linear, bandwidth_linear, order=parameters['order_linear'])
        #linear gain
        g=10**(parameters['g_p0']+parameters['g_m']*log10(cf))
        func_gain=lambda x:g*x
        gain= FunctionFilterbank(gammatone,func_gain)
        #low pass filter(cascade of 4 second order lowpass butterworth filters)
        cutoff_frequencies_linear=10**(parameters['lp_lin_cutoff_p0']+parameters['lp_lin_cutoff_m']*log10(cf))
        order_lowpass_linear=2
        lp_l=LowPass(gain,cutoff_frequencies_linear)
        lowpass_linear=Cascade(gain,lp_l,4)
        
        #### Nonlinear Pathway ####
        #bandpass filter (third order gammatone filters)
        cf_nonlinear=10**(parameters['cf_nl_p0']+parameters['cf_nl_m']*log10(cf))
        bandwidth_nonlinear=10**(parameters['bw_nl_p0']+parameters['bw_nl_m']*log10(cf))
        bandpass_nonlinear1=ApproximateGammatone(source, cf_nonlinear, bandwidth_nonlinear, order=parameters['order_nonlinear'])
        #compression (linear at low level, compress at high level)
        a=10**(parameters['a_p0']+parameters['a_m']*log10(cf))  #linear gain
        b=10**(parameters['b_p0']+parameters['b_m']*log10(cf))  
        v=10**(parameters['c_p0']+parameters['c_m']*log10(cf))#compression exponent
        func_compression=lambda x:sign(x)*minimum(a*abs(x),b*abs(x)**v)
        compression=FunctionFilterbank(bandpass_nonlinear1,  func_compression)
        #bandpass filter (third order gammatone filters)
        bandpass_nonlinear2=ApproximateGammatone(compression, cf_nonlinear, bandwidth_nonlinear, order=parameters['order_nonlinear'])
        #low pass filter
        cutoff_frequencies_nonlinear=10**(parameters['lp_nl_cutoff_p0']+parameters['lp_nl_cutoff_m']*log10(cf))
        order_lowpass_nonlinear=2
        lp_nl=LowPass(bandpass_nonlinear2,cutoff_frequencies_nonlinear)
        lowpass_nonlinear=Cascade(bandpass_nonlinear2,lp_nl,3)
        #adding the two pathways
        drnl_filter=lowpass_linear+lowpass_nonlinear

        self.set_output(drnl_filter)

    
if __name__ == '__main__':        
    
    from brian import *
    set_global_preferences(usenewbrianhears=True,
                           useweave=False)
    from brian.hears import *

    dBlevel=60*dB  # dB level in rms dB SPL
    sound=Sound.load('/home/bertrand/Data/Toolboxes/AIM2006-1.40/Sounds/aimmat.wav')
    samplerate=sound.samplerate
    sound=sound.atlevel(dBlevel)
    
    
    simulation_duration=len(sound)/samplerate
    
    nbr_center_frequencies=50
    center_frequencies=log_space(100*Hz, 1000*Hz, nbr_center_frequencies)
    dnrl_filter=DRNL(sound,center_frequencies)
    dnrl_filter.buffer_init()
    dnrl=dnrl_filter.buffer_fetch(0, len(sound))


'''
Here is defined the main Synapses code.

Two important things:

Synapse object

EventSynapseUpdater

I tried to write as much as I could in the classdoc of each object.

'''

from brian import *
import numpy as np
from brian.inspection import *
from brian.equations import *
from brian.utils.documentation import flattened_docstring
from dev.ideas.synapses.statevectors import *
from dev.ideas.synapses.spikequeue import *
import re

__all__ = ['Synapses']

class Synapses(NetworkOperation):
    '''
    Synapses object
   
     State:
    - the model kwdarg can only delare variables for now
    - only the 'pre' code is implemented 
    - multiple synapses (between same pair of neurons), with multiple delays seem to work fine!
    
    TODO:
    - postsynaptic delays


    ** Initialization **
    
    synapses = syn.Synapses(gin, gout, model = 'w : 1', pre = 'v += w', post = '')
    
    * Arguments * 
    
    ``gin'' source NeuronGroup
    ``gout'' target NeuronGroup
    
    * Keywords *
    
    ``model'' for now only declares variables
    ``pre'' code to be executed when a presynaptic spike arrives
    ``post'' code to be executed when a postsynaptic spike arrives
    
    ``max_delay'' If not set then the synapse is without delays, otherwise sets the maximum acceptable delay for the Synapses. Note that the delays are only axonal (i.e. presynaptic), this could easily be changed in the future.
    
    
    * Attributes *
    
    All the variables of the model (see below), plus
    
    ``existing_synapses'' a 2 D array of shape (2, len(synapses)) that hold the pre and post neuron numbers for each synapse.
    
    
    * Adding Synapses *
    
    When it's created, a Synapses object doesn't have any instantiated synapses. To add some, one must do:
    
    synapses[:,0] = True
    
    Which connects all the presynaptic neurons to the postsynaptic neuron
    
    Alternatively, one may want to add 2 synapses between the same neurons, to do so, one can run the above code line a few times, of just write 
    synapses[:,0] = 2
   
    * Setting variables * 
    
    All of the variables defined in the model code (except the reserved ones), can (and should) be instantiated. In the above example, to set the weights, one can do:
    synapses.w[:,0] = 1 (or array, etc...)
    
    To set all the weights of the *EXISTING* synapses to 1.
    
    If there are multiple synapses, then they can be accessed with a third field:
    synapses.w[:,0,1] = 2
    By default the previous call sets all synapses to the same value.
    
    * Delays * 
    
    If the delays are activated (setting the kwdarg max_delay) then a field of delays (``delay'') is created and can be used as any other field by setting it:

    synapses.delay[:,0] = 5*ms

    '''
    def __init__(self, *args, **kwdargs): # aargh!!
        clock = kwdargs.get('clock', None)
        NetworkOperation.__init__(self, lambda:None, clock=clock)

        # sorry this code might be a bit of a mess.
        # Arguments parsing
        if len(args) == 1 and isinstance(args[0], NeuronGroup):
            self.source = self.target = args[0]
        elif len(args) == 2 and isinstance(args[0], NeuronGroup) and isinstance(args[1], NeuronGroup):
            self.source = args[0]
            self.pre_len = len(args[0])
            self.target = args[1]
            self.post_len = len(args[0])
        else:
            raise ValueError('A Synapse object must be instantiated with one or two NeuronGroups as arguments')
        
        ## KWD arguments parsing
        max_delay = kwdargs.get('max_delay', 0)


        ########### CODE PARSING
        # model equations parsing
        level = kwdargs.get('level', 0)
        model = kwdargs.get('model', None)
        if isinstance(model, Equations):
            model_obj = model
        else:
            if '\n' in model:
                model = flattened_docstring(model)
            elif ';' in model:
                model = '\n'.join([line.strip() for line in model.split(';')])
            model_obj = Equations(model, level = level + 1)
            
        # stolen from NeuronGroup
        # !! This will not work with postsynaptic variables in the model eqs !!
        unit_checking = kwdargs.get('check_units', True)
        method = kwdargs.get('method', None)
        freeze = kwdargs.get('freeze', False)
        implicit = kwdargs.get('implicit', False)
        order = kwdargs.get('order', 1)
        if isinstance(model_obj, StateUpdater):
            self._state_updater = model_obj
            self._all_units = defaultdict() # what is that
        elif isinstance(model_obj, Equations):
            self._eqs = model_obj
            self._state_updater, var_names = magic_state_updater(model_obj, clock=clock, order=order,
                                                                 check_units=unit_checking, implicit=implicit,
                                                                 compile=compile, freeze=freeze,
                                                                 method=method)
            self._state_updater_varnames = var_names
            
            
        # NOW WHAT?!!!!!            
        # I think I should call the state updater with StateVector.states
        # TODO: 
        # - Maybe stateupdaters should be aware of the dtype of the data (or else my state vectors thing is sort of useless
        # - or ensure fast access to different data types in a vectorized way?
            
        # pre/post code parsing
        pre = kwdargs.get('pre', '')
        post = kwdargs.get('post', '')
        # handle multi-line pre, post equations and multi-statement equations separated by ;
        if '\n' in pre:
            pre = flattened_docstring(pre)
        elif ';' in pre:
            pre = '\n'.join([line.strip() for line in pre.split(';')])
        if '\n' in post:
            post = flattened_docstring(post)
        elif ';' in post:
            post = '\n'.join([line.strip() for line in post.split(';')])

        # Check units
        model_obj.compile_functions()
        #model_obj.check_units()
        # Get variable names
        self.vars = model_obj._diffeq_names # !! there are also static variables and aliases !!
        
        self.n_vars = len(self.vars) # ! not very useful

        ############# Setting up the data structure
        # Mandatory fields: 3 for pre/post/delay_pre, all int32 TODO: finer pick of dtype!
        dtypes = (np.int32, ) * 3
        default_labels = ['_pre', '_post', 'delay']
        # Equation defined fields (float32)
        dtypes += (np.float32, ) * self.n_vars

        # construction of the structure
        self._statevector = ConstructionSparseStateVector(len(dtypes), dtype = dtypes, labels = default_labels+self.vars)
        

        ############# Code!!!
        # create namespace
        pre_namespace = namespace(pre, level = level + 1)
        pre_namespace['target'] = self.target
        pre_namespace['unique'] = np.unique
        pre_namespace['nonzero'] = np.nonzero
        pre_namespace['_pre'] = self._statevector._pre
        pre_namespace['_post'] = self._statevector._post

        
        for var in self.vars:
            pre_namespace[var] = self._statevector[var]
        # !! also add postsynaptic variables !!

        def update_code(pre, indices):
            res = pre
            # given the synapse indices, write the update code,
            # this is here because in the code we generate we need to write this twice (because of the multiple presyn spikes for the same postsyn neuron problem)
            for postsyn_var in self.target.var_index:
                if isinstance(postsyn_var, str):
                    print 'postsyn', postsyn_var
                    res = re.sub(r'\b' + postsyn_var + r'\b', 'target.' + postsyn_var + '[_post['+indices+']]', res)# postsyn variable, indexed by post syn neuron numbers
                
            for var in self.vars:
                res = re.sub(r'\b' + var + r'\b', var + '['+indices+']', res) # synaptic variable, indexed by the synapse number
            return res
 
        # pre code
        pre_code = "_post_neurons = _post[_synapses]\n"
        # which post syn neurons
        pre_code += "_u, _i = unique(_post_neurons, return_index = True)\n"
        pre_code += update_code(pre, '_synapses[_i]') + "\n"
        pre_code += "if len(_u) < len(_post_neurons):\n"
        pre_code += "    _post_neurons[_i] = -1\n"
        pre_code += "    while (len(_u) < len(_post_neurons)) & (_post_neurons>-1).any():\n" # !! the any() is time consuming (len(u)>=1??)
        pre_code += "        _u, _i = unique(_post_neurons, return_index = True)\n"
        pre_code += "        " + update_code(pre, '_synapses[_i[1:]]') + "\n"
        pre_code += "        _post_neurons[_i[1:]] = -1 \n"
        log_debug('brian.synapses', '\nPRE CODE:\n'+pre_code)
        
        pre_code = compile(pre_code, "Presynaptic code", "exec")
        
        
        self.pre_namespace = pre_namespace
        self.pre_code = pre_code
        self.pre_queue = SpikeQueue(self.source, self, max_delay = max_delay)

        self.contained_objects = [self.pre_queue] # wtf is this for      
        
    def __setitem__(self, key, value):
        if not isinstance(key, tuple):
            # do they?
            raise ValueError('Synapses behave as 2-D objects')

        if isinstance(value, (int, bool)):
            # Simple case, either one or multiple synapses between different neurons
            if not (value is True):
                n_synapses = value
            else: # !! and False? !!
                n_synapses = 1
        
            pre_slice = slice2range(key[0], len(self.source))
            post_slice = slice2range(key[1], len(self.target))
        
            # Construction of the pre/post neurons indices list
            # This needs speed up!!!!
            # BTW not sure that lists are really necessary, maybe at build time though
            pre, post = [], []
            for i in pre_slice:
                pre += [i]*len(post_slice)*n_synapses # transposed?
                post += list(post_slice)*n_synapses
        elif isinstance(value, str):
            # !! Other more efficient option for memory consumption: !!
            # vectorize only over post neurons, loop over pre
            # transform boolean to sparse vector after every pre neuron
            #
            # more complex case where the value is specified as a string, hence code
            # The code is evaluated and this yields a matrix that is then passed 
            code = re.sub(r'\b' + 'rand\(\)', 'rand(len(_i), len(_j))', value) # replacing rand()

            pre_slice = slice2range(key[0], len(self.source))
            post_slice = slice2range(key[1], len(self.target))

            pre_grid, post_grid = np.meshgrid(pre_slice, post_slice)
            
            namespace = {'_i' : pre_slice,
                         '_j' : post_slice,
                         'i' : pre_grid,
                         'j' : post_grid,
                         'rand': np.random.rand}
            log_debug('synapses.setitem', code)
            
            res = eval(code, namespace)
            
            log_debug('synapses.setitem', 'created synapses: \n '+ str(res))
            
            self[key] = res
            return
        elif isinstance(value, np.ndarray):
            n_syn = array(value.copy(), dtype = int)
            pre, post = np.nonzero(n_syn)
            if (n_syn >= 1).any():
                # no need to go through this if there is never multiple synapses
                pre = list(pre)
                post = list(post)
                n_syn -= 1
                n_syn[n_syn < 0] = 0
                _pre, _post = np.nonzero(n_syn)
                while len(_pre) != 0:
                    pre += list(_pre)
                    post += list(_post)
                    n_syn -= 1
                    n_syn[n_syn < 0] = 0
                    _pre, _post = np.nonzero(n_syn)
        # !! add: sparse matrices
        
        total_n = len(pre)
        # values for the other fields (0)
        pre = np.array(pre, dtype = np.int32)
        post = np.array(post, dtype = np.int32)
        delays = np.zeros(total_n, dtype = np.int32)
        
        values = [pre, post, delays]
        values += [np.zeros(total_n, dtype = typ) for typ in self._statevector.dtypes[3:]]

        self._statevector.append(values)
        
        return total_n

    def __getattr__(self, name):
        if hasattr(self, '_statevector'):
            if name == '_S':
                # for the state updater
                return self._statevector._allstates
            if name in self._statevector.labels:
                # Have to return a special kind of Vector, for the slicing to work
                data = getattr(self._statevector, name)
                groups_shape = (self.pre_len, self.post_len)

                # dt handling (see the ParameterVector doc)
                dt = None
                if name == 'delay':
                    dt = float(self.source.clock.dt)

                return ParameterVector(data, groups_shape, self.existing_synapses, delay_dt = dt)
        try:
            self.__dict__[name]
        except KeyError:
            raise AttributeError('Synapses object doesn\'t have a '+name+' attribute')
        
    
    def __len__(self):
        '''
        Returns the number of existing synapses.
        '''
        return self._statevector.nvalues
    
    def __call__(self):
        # presynaptic spikes
        synapses_current = self.pre_queue.peek()
        if len(synapses_current):
            _namespace = self.pre_namespace
            _namespace['_synapses'] = np.array(synapses_current)
            for var in self.vars:
                _namespace[var] = self._statevector[var]
            _namespace['t'] = self.source.clock._t
            _namespace['_post'] = self._statevector._post
            exec self.pre_code in _namespace
            
        self.pre_queue.next()
        
        # state update
        self._state_updater(self)

    @property
    def existing_synapses(self):
        return np.vstack((self._statevector._pre, self._statevector._post))
    
    def __repr__(self):
        s = 'Synapses:\n'
        s+= str(self.existing_synapses)
        return s

# DEPRECATED STUFF:
#
# ########################## SynapseUpdater ##########################

# class EventSynapseUpdater(SpikeMonitor):
#     '''
#     Mimicked after the eventstdpupdater in dev/ideas/stdp/eventdriven_stdp
    
#     A SpikeMonitor that executes the code it's given at initialization.
    
#     The interesting code is rather in the Synapses object itself (search for pre_code)
#     '''
#     def __init__(self, source, S, code, namespace, delay=0 * ms):
#         '''
#         source = source group
#         S = base Synapse object
#         vars = variable names
#         M = matrix of the linear differential system
#         code = code to execute for every spike
#         namespace = namespace for the code
#         delay = transmission delay 
#         '''
#         super(EventSynapseUpdater, self).__init__(source, 
#                                                   record = False)
#         self._code = code # update code
#         self._namespace = namespace # code namespace
#         self.S = S
        
#     def propagate(self, spikes):
#         if len(spikes):
#             synapses = []
#             for i in spikes:
#                 synapses += list(nonzero(self.S._statevector._pre == i)[0])
                
#             self._namespace['_synapses'] = np.array(synapses)
#             for var in self.S.vars:
#                 self._namespace[var] = self.S._statevector[var]
#             self._namespace['t'] = self.S.source.clock._t
#             self._namespace['_post'] = self.S._statevector._post
#             exec self._code in self._namespace


# INTIAL_MAXSPIKESPER_DT = 10 # Because SpikeQueue doesn't have a dynamic structure (yet)
# class DelayEventSynapseUpdater(SpikeMonitor):
#     '''
#     Same as above but includes delays (yay!)
#     '''
#     def __init__(self, source, S, code, namespace, max_delay):
#         '''

#         source = source group
#         S = base Synapse object
#         vars = variable names
#         M = matrix of the linear differential system
#         code = code to execute for every spike
#         namespace = namespace for the code

#         max_delay = maximum delay
#         '''
#         super(DelayEventSynapseUpdater, self).__init__(source, 
#                                                   record = False)
#         self._code = code # update code
#         self._namespace = namespace # code namespace
#         self.S = S
#         # SpikeQueue initialization
#         nsteps = int(np.ceil((max_delay)/(self.S.source.clock.dt)))
#         self.spike_queue = SpikeQueue(source, INTIAL_MAXSPIKESPER_DT)
        
#     def propagate(self, spikes):
#         # step 1: insert spikes in the event queue           
#         if len(spikes):
#             # synapse identification, 
#             # this seems ok in terms of speed even though I dont like the for loop. 
#             # any idea? see stest_fastsynapseidentification.py
#             synapses = []
#             for i in spikes:
#                 synapses += list(nonzero(self.S._statevector._pre == i)[0]) 


#             # delay getting:
#             delays = self.S._statevector.delay[synapses]
#             offsets = self.spike_queue.offsets(delays)
#             self.spike_queue.insert(delays, offsets, synapses)
            
#         # step 2: execute code on current spikes
#         synapses_current = self.spike_queue.peek()
#         if len(synapses_current):
#             self._namespace['_synapses'] = np.array(synapses_current)
#             for var in self.S.vars:
#                 self._namespace[var] = self.S._statevector[var]
#             self._namespace['t'] = self.S.source.clock._t
#             self._namespace['_post'] = self.S._statevector._post
#             exec self._code in self._namespace
            
#         self.spike_queue.next()



from brian import *
from brian.library.modelfitting import *

if __name__ == '__main__':
    
    duration = 1*second

    model = Equations('''
        dV/dt=(R*I-V)/tau : 1
        I : 1
        R : 1
        tau : second
    ''')

    input = randn(int(duration/defaultclock.dt))*2e-10+4e-10
    
    def getspikes():
        G = NeuronGroup(1, model, reset=0, threshold=1, refractory=5*ms)
        G.R = 3e9
        G.tau = 20*ms
        M = SpikeMonitor(G)
        G.I = TimedArray(input)
        run(duration)
        i, t = zip(*M.spikes)
        clear(True)
        reinit_default_clock()
        return t
    
    spikes0 = getspikes()
    spikes = []
    for i in xrange(1):
        spikes.extend([(i, spike + 5*i*ms) for spike in spikes0])

    results = modelfitting( model = model,
                            reset = 0,
                            threshold = 1,
                            data = spikes,
                            input = input,
                            dt = .1*ms,
                            popsize = 150000,
                            maxiter = 50,
                            gpu = 1,
                            algorithm = CMAES,
                            delta = 1*ms,
                            R = [1.0e9, 9.0e9],
                            tau = [10*ms, 40*ms],
                            delays = [-20*ms, 20*ms],
                            refractory=[0*ms, 0*ms, 10*ms, 10*ms])
    print_table(results)



from brian import Equations, NeuronGroup, Clock, CoincidenceCounter, Network, zeros, array, \
                    ones, kron, ms, second, concatenate, hstack, sort, nonzero, diff, TimedArray, \
                    reshape, sum, log, Monitor, NetworkOperation, defaultclock, linspace, vstack, \
                    arange, sort_spikes, rint, SpikeMonitor, Connection, Threshold, Reset, \
                    int32, double, VariableReset, StringReset, VariableThreshold, StringThreshold, \
                    Refractoriness
from brian.tools.statistics import firing_rate, get_gamma_factor
from playdoh import *
import brian.optimiser as optimiser
import pycuda.driver as drv
import pycuda
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
try:
    from pycuda.compiler import SourceModule
except ImportError:
    from pycuda.driver import SourceModule
from numpy import *
from brian.experimental.codegen.integration_schemes import *
from brian.experimental.codegen.codegen_gpu import *
import re

__all__ = ['GPUModelFitting',
           'euler_scheme', 'rk2_scheme', 'exp_euler_scheme','close_cuda'
           ]

if drv.get_version() == (2, 0, 0): # cuda version
    default_precision = 'float'
elif drv.get_version() > (2, 0, 0):
    default_precision = 'double'
else:
    raise Exception, "CUDA 2.0 required"

class ModelfittingGPUCodeGenerator(GPUCodeGenerator):
    def generate(self, eqs, scheme):
        vartype = self.vartype()
        code = ''
        for line in self.scheme(eqs, scheme).split('\n'):
            line = line.strip()
            if line:
                code += '        ' + line + '\n'
        return code




BLOCKSIZE = 256

def get_cuda_template():
    return """
__global__ void runsim(
    // ITERATIONS
    int Tstart, int Tend,             // Start, end time as integer (t=T*dt)
    int duration,                     // Total duration as integer
    
    // STATE VARIABLES
    %SCALAR% *state_vars,             // State variables are offset from this
    
    // INPUT PARAMETERS
    double *I_arr,                    // Input current
    int *I_arr_offset,                // Input current offset (for separate input
                                      // currents for each neuron)
    // DELAYS PARAMETERS
    int *spikedelay_arr,              // Integer delay for each spike
    
    // REFRACTORY PARAMETERS
    int *refractory_arr,              // Integer refractory times
    int *next_allowed_spiketime_arr,  // Integer time of the next allowed spike (for refractoriness)
    
    // CRITERION SPECIFIC PARAMETERS
    %CRITERION_DECLARE%
    
    // DATA DECLARE
    %DATA_DECLARE%
    
    // STATEMONITOR DECLARE
    %STATEMONITOR_DECLARE%
    
    // SPIKEMONITOR DECLARE
    %SPIKEMONITOR_DECLARE%
    
    // MISC PARAMETERS
    int onset                         // Time onset (only count spikes from here onwards)
    )
{
    // NEURON INDEX
    const int neuron_index = blockIdx.x * blockDim.x + threadIdx.x;
    if(neuron_index>=%NUM_NEURONS%) return;
    
    // EXTRACT STATE VARIABLES
    %EXTRACT_STATE_VARIABLES%
    
    // LOAD VARIABLES
    %LOAD_VARIABLES%
    
    // DATA INIT
    %DATA_INIT%
    
    // STATEMONITOR INIT
    %STATEMONITOR_INIT%
    
    // SPIKEMONITOR INIT
    %SPIKEMONITOR_INIT%
    
    // CRITERION INITIALIZATION
    %CRITERION_INIT%
    
    // INPUT INITIALIZATION
    int I_offset0 = I_arr_offset[blockIdx.x * blockDim.x]; // I_offset of the first thread in the block
    int I_offset = I_arr_offset[neuron_index];
    
    // DELAYS INITIALIZATION
    int spikedelay = spikedelay_arr[neuron_index];
    
    // REFRACTORY INITIALIZATION
    const int refractory = refractory_arr[neuron_index];
    int next_allowed_spiketime = next_allowed_spiketime_arr[neuron_index];
    
    %SCALAR% ${input_var} = 0;
    
    for(int T=Tstart; T<Tend; T++)
    {
        %SCALAR% t = T*%DT%;
        
        ${input_var} = I_arr[T+I_offset];
        
        // STATE UPDATE
        // Update the variables only for T>=onset for neurons in the first group
        if ((neuron_index>=%NUM_NEURONS_FIRSTGROUP%)|(T>=onset))
        {
            %STATE_UPDATE%
        }
        
        // THRESHOLD
        const bool is_refractory = (T<=next_allowed_spiketime);
        const bool has_spiked = (%THRESHOLD%)&&!is_refractory;
        
        // RESET
        if(has_spiked||is_refractory)
        {
            %RESET1%
        }

        if(has_spiked)
        {
            next_allowed_spiketime = T+refractory;
            %RESET2%
            %SPIKEMONITOR_UPDATE%
        }
        // DATA UPDATE
        %DATA_UPDATE%
        
        // STATEMONITOR UPDATE
        %STATEMONITOR_UPDATE%
        
        // CRITERION TIMESTEP
        %CRITERION_TIMESTEP%
        
    }
    // STORE VARIABLES
    %STORE_VARIABLES%
    
    // STATEMONITOR END
    %STATEMONITOR_END%
    
    // SPIKEMONITOR END
    %SPIKEMONITOR_END%
    
    // CRITERION END
    %CRITERION_END%
    
    next_allowed_spiketime_arr[neuron_index] = next_allowed_spiketime;
    
    // END
    %DATA_END%
}
    """




class GPUModelFitting(object):
    '''
    Model fitting class to interface with GPU
    
    Initialisation arguments:
    
    ``G``
        The initialised NeuronGroup to work from.
    ``eqs``
        The equations defining the NeuronGroup.
    ``I``, ``I_offset``
        The current array and offsets (see below).
    ``spiketimes``, ``spiketimes_offset``
        The spike times array and offsets (see below).
    ``spikedelays``,
        Array of delays for each neuron.
    ``refractory``,
        Array of refractory periods, or a single value.
    ``delta``
        The half-width of the coincidence window.
    ``precision``
        Should be 'float' or 'double' - by default the highest precision your
        GPU supports.
    ``coincidence_count_algorithm``
        Should be 'inclusive' if multiple predicted spikes can match one
        target spike, or 'exclusive' (default) if multiple predicted spikes
        can match only one target spike (earliest spikes are matched first). 
    
    Methods:
    
    ``reinit_vars(I, I_offset, spiketimes, spiketimes_offset, spikedelays)``
        Reinitialises all the variables, counters, etc. The state variable values
        are copied from the NeuronGroup G again, and the variables I, I_offset, etc.
        are copied from the method arguments.
    ``launch(duration[, stepsize])``
        Runs the kernel on the GPU for simulation time duration. If ``stepsize``
        is given, the simulation is broken into pieces of that size. This is
        useful on Windows because driver limitations mean that individual GPU kernel
        launches cannot last more than a few seconds without causing a crash.
    
    Attributes:
    
    ``coincidence_count``
        An array of the number of coincidences counted for each neuron.
    ``spike_count``
        An array of the number of spikes counted for each neuron.
    
    **Details**
    
    The equations for the NeuronGroup can be anything, but they will be solved
    with the Euler method. One restriction is that there must be a parameter
    named I which is the time varying input current. (TODO: support for
    multiple input currents? multiple names?)
    
    The current I is passed to the GPU in the following way. Define a single 1D
    array I which contains all the time varying current arrays one after the
    other. Now define an array I_offset of integers so that neuron i should see
    currents: I[I_offset[i]], I[I_offset[i]+1], I[I_offset[i]+2], etc.
    
    The experimentally recorded spike times should be passed in a similar way,
    put all the spike times in a single array and pass an offsets array
    spiketimes_offset. One difference is that it is essential that each spike
    train with the spiketimes array should begin with a spike at a
    large negative time (e.g. -1*second) and end with a spike that is a long time
    after the duration of the run (e.g. duration+1*second). The GPU uses this to
    mark the beginning and end of the train rather than storing the number of
    spikes for each train. 
    '''
    def __init__(self,
                 G,
                 eqs,
                 criterion, # Criterion object
                 input_var,
                 subpopsize,
                 onset=0*ms,
                 precision=default_precision,
                 statemonitor_var=None,
                 spikemonitor = False,
                 nbr_spikes = 200,
                 duration=None,
                 scheme=euler_scheme,
                 stand_alone=False
                 ):
        eqs.prepare()
        self.precision = precision
        self.scheme = scheme
        if precision == 'double':
            self.mydtype = float64
        else:
            self.mydtype = float32
        self.stand_alone=stand_alone
        self.N = len(G)
        self.dt = G.clock.dt
        self.onset = onset
        self.eqs = eqs
        self.G = G
        self.spikemonitor = spikemonitor
        self.nbr_spikes = nbr_spikes
        self.subpopsize = subpopsize
        self.duration = int(ceil(duration/self.dt))
        self.input_var = input_var
        self.statemonitor_var = statemonitor_var
        self.criterion = criterion
        self.generate_code()

    def generate_threshold_code(self, src):
        eqs = self.eqs
        threshold = self.G._threshold
        if threshold.__class__ is Threshold:
            state = threshold.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            threshold = state + '>' + str(float(threshold.threshold))
        elif isinstance(threshold, VariableThreshold):
            state = threshold.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            threshold = state + '>' + threshold.threshold_state
        elif isinstance(threshold, StringThreshold):
            namespace = threshold._namespace
            expr = threshold._expr
            all_variables = eqs._eq_names + eqs._diffeq_names + eqs._alias.keys() + ['t']
            expr = optimiser.freeze(expr, all_variables, namespace)
            threshold = expr
        else:
            raise ValueError('Threshold must be constant, VariableThreshold or StringThreshold.')
        
        # Substitute threshold
        src = src.replace('%THRESHOLD%', threshold)
        
        return src
     
    def generate_reset_code(self, src):
        eqs = self.eqs
        reset = self.G._resetfun
#        print eqs
        if reset.__class__ is Refractoriness:
            state = reset.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            reset = state + ' = ' + str(float(reset.resetvalue))
        elif reset.__class__ is Reset:
            state = reset.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            reset = state + ' = ' + str(float(reset.resetvalue))
        elif isinstance(reset, VariableReset):
            state = reset.state
            if isinstance(state, int):
                state = eqs._diffeq_names[state]
            reset = state + ' = ' + reset.resetvaluestate
        elif isinstance(reset, StringReset):
            namespace = reset._namespace
            expr = reset._expr
            all_variables = eqs._eq_names + eqs._diffeq_names + eqs._alias.keys() + ['t']
            expr = optimiser.freeze(expr, all_variables, namespace)
            reset = expr

#        self.reset = reset
        # Substitute reset
        reset = '\n            '.join(line.strip() + ';' for line in reset.split('\n') if line.strip())
        reset=reset.split('\n')
        src = src.replace('%RESET1%', reset[0])
        reset = '\n            '.join(line.strip()+'\n' for line in reset[1:])
        src = src.replace('%RESET2%', reset)
#        print src
        return src
            
    def generate_data_code(self, src):
        # Substitute spikes/traces declare
        if self.criterion.type == 'spikes':
            data_declare = """
    int *spiketimes,          // Array of all spike times as integers (begin and
                              // end each train with large negative value)
    int *spiketime_indices,   // Pointer into above array for each neuron
            """
        if self.criterion.type == 'traces':
            data_declare = """
    double *traces_arr,
    int *traces_arr_offset,
            """
        src = src.replace('%DATA_DECLARE%', data_declare)
        
        # Substitute spikes/traces init
        if self.criterion.type == 'spikes':
            data_init = """
    int spiketime_index = spiketime_indices[neuron_index];
            """
        if self.criterion.type == 'traces':
            data_init = """
    int trace_offset = traces_arr_offset[neuron_index];
    double trace_value = 0.0;
    int Tdelay = 0;
            """
        src = src.replace('%DATA_INIT%', data_init)
        
        # Substitute spikes/traces update
        if self.criterion.type == 'spikes':
            data_update = """
            """
        if self.criterion.type == 'traces':
            data_update = """
        Tdelay = T+spikedelay;
        if ((Tdelay>=0)&(Tdelay<duration-1)) {
            trace_value = traces_arr[Tdelay+trace_offset+1];
        }
            """
        src = src.replace('%DATA_UPDATE%', data_update)
        
        # Substitute spikes/traces end
        if self.criterion.type == 'spikes':
            data_end = """
            spiketime_indices[neuron_index] = spiketime_index;
            """
        if self.criterion.type == 'traces':
            data_end = """
            """
        src = src.replace('%DATA_END%', data_end)
        
        return src
    
    def generate_spikemonitor_code(self, src):
        if self.spikemonitor is True:
            declare = "\n".join(["%SCALAR% *spikemonitor_container,"])
            declare += """
                    int *spikemonitor_offsets,
                        """
            init = """
                    int spikemonitor_offset = spikemonitor_offsets[neuron_index];
                    """
            update = "".join('spikemonitor_container[spikemonitor_offset] = T+spikedelay;')
            update += """
                    spikemonitor_offset++;
                    """ 
            end = """
                spikemonitor_offsets[neuron_index] = spikemonitor_offset;
                """
        else:
            declare = ""
            init = ""
            update = ""
            end = ""
        src = src.replace('%SPIKEMONITOR_DECLARE%', declare)
        src = src.replace('%SPIKEMONITOR_INIT%', init)
        src = src.replace('%SPIKEMONITOR_UPDATE%', update)
        src = src.replace('%SPIKEMONITOR_END%', end)
        return src
        
    def generate_statemonitor_code(self, src):
        if self.statemonitor_var is not None:
            declare = "\n".join(["%SCALAR% *statemonitor_"+state+"_values," for state in self.statemonitor_var])
            declare += """
    int *statemonitor_offsets,
            """
            init = """
    int statemonitor_offset = statemonitor_offsets[neuron_index];
            """
            update = "".join([("""
        statemonitor_%s_values[statemonitor_offset] = %s;
        """ % (state,state)) for state in self.statemonitor_var])
            update += """
        statemonitor_offset++;
            """ % self.statemonitor_var
            end = """
    statemonitor_offsets[neuron_index] = statemonitor_offset;
            """
        else:
            declare = ""
            init = ""
            update = ""
            end = ""
        src = src.replace('%STATEMONITOR_DECLARE%', declare)
        src = src.replace('%STATEMONITOR_INIT%', init)
        src = src.replace('%STATEMONITOR_UPDATE%', update)
        src = src.replace('%STATEMONITOR_END%', end)
        return src
    
    def generate_code(self):
        self.eqs.prepare()
        src = get_cuda_template()
        # Substitute state variable declarations
        indexvar = dict((v, k) for k, v in self.G.var_index.iteritems() if isinstance(k, str) and k!='I')
        extractions = '\n    '.join('%SCALAR% *'+name+'_arr = state_vars+'+str(i*self.N)+';' for i, name in indexvar.iteritems())
        src = src.replace('%EXTRACT_STATE_VARIABLES%', extractions)
        #here
        # Substitute load variables
        loadvar_names = self.eqs._diffeq_names + []
        loadvar_names.remove('I') # I is assumed to be a parameter and loaded per time step
        loadvars = '\n    '.join('%SCALAR% ' + name + ' = ' + name + '_arr[neuron_index];' for name in loadvar_names)
        
        # simple equation patterns
        for name in self.eqs._string.keys():
            if name not in self.eqs._diffeq_names:
                loadvars += '\n    %SCALAR% ' + name + ' = 0;'
        
        src = src.replace('%LOAD_VARIABLES%', loadvars)
        
        # Substitute save variables
        savevars = '\n    '.join(name + '_arr[neuron_index] = ' + name + ';' for name in loadvar_names)
        src = src.replace('%STORE_VARIABLES%', savevars)
        
        src = self.generate_threshold_code(src)
        src = self.generate_reset_code(src)
        src = self.generate_data_code(src)
        src = self.generate_statemonitor_code(src)
        src = self.generate_spikemonitor_code(src)
        # Substitute state update
        sulines = ModelfittingGPUCodeGenerator(dtype=self.precision).generate(self.eqs, self.scheme)
        
        # simple equation patterns
        for name in self.eqs._string.keys():
            if name not in self.eqs._diffeq_names:
                sulines += '        ' + name + ' = ' + self.eqs._string[name] + ';\n'
        
        sulines = re.sub(r'\bdt\b', '%DT%', sulines)
        src = src.replace('%STATE_UPDATE%', sulines.strip())
        
        # Substitute criterion code
        criterion_code = self.criterion.get_cuda_code()
        for search, replace in criterion_code.iteritems():
            src = src.replace(search, replace)
        
        # Substitute dt
        src = src.replace('%DT%', str(float(self.dt)))
        # Substitute SCALAR
        src = src.replace('%SCALAR%', self.precision)
        # Substitute number of neurons
        src = src.replace('%NUM_NEURONS%', str(self.N))
        src = src.replace('%NUM_NEURONS_FIRSTGROUP%', str(self.subpopsize))
        src = src.replace('%BLOCKSIZE%', str(BLOCKSIZE))
        # Substitute input var name
        src = src.replace('${input_var}', str(self.input_var))
        #print src
        #return
        self.kernel_src = src

    def initialize_spikes(self, spiketimes, spiketimes_indices):
        self.spiketimes = gpuarray.to_gpu(array(rint(spiketimes / self.dt), dtype=int32))
        self.spiketime_indices = gpuarray.to_gpu(array(spiketimes_indices, dtype=int32))
        
    def initialize_traces(self, traces, traces_offset):
        self.traces = gpuarray.to_gpu(array(traces, dtype=double))
        self.traces_offset = gpuarray.to_gpu(array(traces_offset, dtype=int32))

    def initialize_delays(self, spikedelays):
        self.spikedelay_arr = gpuarray.to_gpu(array(rint(spikedelays / self.dt), dtype=int32))
    
    def initialize_refractory(self, refractory):
        if isinstance(refractory, float):
            refractory = refractory*ones(self.N)
        self.refractory_arr = gpuarray.to_gpu(array(rint(refractory / self.dt), dtype=int32))
        self.next_allowed_spiketime_arr = gpuarray.to_gpu(-ones(self.N, dtype=int32))

    def initialize_statemonitor(self):
        self.statemonitor_values = [gpuarray.to_gpu(zeros(self.N*self.duration, 
                                                          dtype=self.precision))
                                    for state in self.statemonitor_var]
        self.statemonitor_offsets = gpuarray.to_gpu(arange(0, self.duration*self.N+1,
                                                           self.duration, 
                                                           dtype=int32))
    def initialize_spikemonitor(self):
        self.spikemonitor_values = gpuarray.to_gpu(zeros(self.N*self.nbr_spikes, 
                                                          dtype=self.precision))
        self.spikemonitor_offsets = gpuarray.to_gpu(arange(0, self.nbr_spikes*self.N+1,
                                                   self.nbr_spikes, 
                                                   dtype=int32))

    
    def initialize_kernel_arguments(self):
        self.kernel_func_args = [self.statevars_arr,
                                 self.I,
                                 self.I_offset,
                                 self.spikedelay_arr,
                                 self.refractory_arr,
                                 self.next_allowed_spiketime_arr]
        self.kernel_func_args += self.criterion.get_kernel_arguments()
        if self.criterion.type == 'spikes':
            self.kernel_func_args += [self.spiketimes,
                                      self.spiketime_indices]
        if self.criterion.type == 'traces':
            self.kernel_func_args += [self.traces,
                                      self.traces_offset]
        if self.statemonitor_var is not None:
            self.kernel_func_args += [val for val in self.statemonitor_values]
            self.kernel_func_args += [self.statemonitor_offsets]
            
        if self.spikemonitor is True:
            self.kernel_func_args += [self.spikemonitor_values,self.spikemonitor_offsets]
            #self.kernel_func_args += [self.spikemonitor_offsets]
        
        self.kernel_func_args += [int32(rint(self.onset / self.dt))]

    def reinit_vars(self, criterion,
                    I, I_offset,
                    spiketimes = None, spiketimes_offset = None,
                    traces = None, traces_offset = None,
                    spikedelays = None, refractory = None):
        self.criterion = criterion
        
        self.kernel_module = SourceModule(self.kernel_src)
        self.kernel_func = self.kernel_module.get_function('runsim')
        blocksize = BLOCKSIZE
#        try:
#            blocksize = self.kernel_func.get_attribute(pycuda.driver.function_attribute.MAX_THREADS_PER_BLOCK)
#        except: # above won't work unless CUDA>=2.2
#            pass
        self.block = (blocksize, 1, 1)
        self.grid = (int(ceil(float(self.N) / blocksize)), 1)
        self.kernel_func_kwds = {'block':self.block, 'grid':self.grid}
        mydtype = self.mydtype
        N = self.N
        eqs = self.eqs
        statevars_arr = gpuarray.to_gpu(array(self.G._S.flatten(), dtype=mydtype))
        self.I = gpuarray.to_gpu(array(I, dtype=mydtype))
        self.statevars_arr = statevars_arr
        
        self.I_offset = gpuarray.to_gpu(array(I_offset, dtype=int32))
        # SPIKES
        if self.criterion.type == 'spikes':
            self.initialize_spikes(spiketimes, spiketimes_offset)
        # TRACES
        if self.criterion.type == 'traces':
            self.initialize_traces(traces, traces_offset)
        self.criterion.initialize_cuda_variables()
        self.initialize_delays(spikedelays)
        self.initialize_refractory(refractory)
        
        if self.statemonitor_var is not None:
            self.initialize_statemonitor()
            
        if self.spikemonitor is True:
            self.initialize_spikemonitor()
        
        self.initialize_kernel_arguments()

    def launch(self, duration, stepsize=128*ms):
        if stepsize is None:
            self.kernel_func(int32(0), int32(duration / self.dt),
                             *self.kernel_func_args, **self.kernel_func_kwds)
            pycuda.context.synchronize()
        else:
            stepsize = int(stepsize / self.dt)
            duration = int(duration / self.dt)
            for Tstart in xrange(0, duration, stepsize):
                Tend = Tstart + min(stepsize, duration - Tstart)
                self.kernel_func(int32(Tstart), int32(Tend), int32(duration),
                                 *self.kernel_func_args, **self.kernel_func_kwds)
                pycuda.context.synchronize()
#        if self.stand_alone and 1:
##            print 'close stand alone'
##            drv.init()
#            pycuda.context.pop()
#            pycuda.context = None
            
    def get_statemonitor_values(self):
        values = [val.get().reshape((self.N, -1)) for val in self.statemonitor_values]
        if len(self.statemonitor_var)==1:
            return values[0]
        else:
            return values
        
    def get_spikemonitor_values(self):
        values = self.spikemonitor_values.get().reshape((self.N, -1))
        return values
        
    def close_cuda():
        """
        Closes the current PyCUDA context. MUST be called at the end of the
        script.
        """
        print 'close'
        log_debug("Trying to close current PyCUDA context")
        if pycuda.context is not None:
            try:
                log_debug("Closing current PyCUDA context")
                pycuda.context.pop()
                pycuda.context = None
            except:
                log_warn("A problem occurred when closing PyCUDA context")






'''
Example of using derived classes in Brian

Using a class derived from one of Brian's classes can be a useful way of
organising code in complicated simulations. A class such as a :class:`NeuronGroup`
can itself create further :class:`NeuronGroup`, :class:`Connection` and
:class:`NetworkOperation` objects. In order to have these objects included in
the simulation, the derived class has to include them in its ``contained_objects``
list (this tells Brian to add these to the :class:`Network` when the derived
class object is added to the network).
'''
from brian import *


class PoissonDrivenGroup(NeuronGroup):
    '''
    This class is a group of leaky integrate-and-fire neurons driven by
    external Poisson inputs. The class creates the Poisson inputs and
    connects them to itself. 
    '''
    def __init__(self, N, rate, weight):
        tau = 10 * ms
        eqs = '''
        dV/dt = -V/tau : 1
        '''
        # It's essential to call the initialiser of the base class
        super(PoissonDrivenGroup, self).__init__(N, eqs, reset=0, threshold=1)
        self.poisson_group = PoissonGroup(N, rate)
        self.conn = Connection(self.poisson_group, self, 'V')
        self.conn.connect_one_to_one(weight=weight)
        self.contained_objects += [self.poisson_group,
                                   self.conn]

G = PoissonDrivenGroup(100, 100 * Hz, .3)

M = SpikeMonitor(G)
M_pg = SpikeMonitor(G.poisson_group)
trace = StateMonitor(G, 'V', record=0)

run(1 * second)

subplot(311)
raster_plot(M_pg)
title('Input spikes')
subplot(312)
raster_plot(M)
title('Output spikes')
subplot(313)
plot(trace.times, trace[0])
title('Sample trace')
show()
